# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os

import wx
import numpy

from .animationtoolbar import AnimationToolbar, EVT_KEYFRAME_CHANGED
from .plotdatasetpanel import PlotDataSetPanel
from .plotpropertypanel import PlotPropertyPanel
from .plotwindow import PlotWindow, EVT_PLOTDATA_CHANGED, EVT_PLOTAXIS_CHANGED
from .typedsavefiledialog import TypedSaveFileDialog

windowCount = 1


class PlotFrame(wx.Frame):

    def __init__(self, parent=None, id=-1, title=None, pos=wx.DefaultPosition, size=(800, 494)):
        if parent is None:
            # use top level wxWindow
            try:
                parent = wx.GetApp().GetTopWindow()
            except Exception:
                # If there is no top level window, just create parentless plot window
                pass
        if title is None:
            # determine title
            global windowCount
            title = "VISAN 2D Plot"
            if windowCount > 1:
                title += " - %d" % windowCount
            windowCount += 1
        super(PlotFrame, self).__init__(parent, id, title, pos, size)

        # Set icon if possible
        if wx.Config.Get().Read("IconFile"):
            self.SetIcon(wx.Icon(wx.Config.Get().Read("IconFile")))

        # Set initial state
        self.closingDown = False
        self.filename = ""
        self.dataSetAttributes = []
        self.dataSetLocation = []

        # Create and configure all widgets
        self.CreateMenuBar()
        self.CreateControls()
        self.CreateLayout()

        # Other Event Listeneres
        self.Bind(wx.EVT_CLOSE, self.OnClose)

        # Give the plotwindow the initial focus (since this might not happen automatically for e.g. Mac OS X)
        self.plotWindow.SetFocus()

        self.Show()
        wx.Yield()

    def __repr__(self):
        return "<visan plot handle>"

    def CreateMenuBar(self):
        menubar = wx.MenuBar()

        filemenu = wx.Menu()

        item = filemenu.Append(wx.ID_SAVE, "&Save Image...\tCtrl-S", "Save the image to an image file")
        self.Bind(wx.EVT_MENU, self.OnSave, item)

        filemenu.AppendSeparator()

        item = filemenu.Append(wx.ID_CLOSE, "&Close\tCtrl-W", "Close this Window")
        self.Bind(wx.EVT_MENU, self.OnClose, item)

        menubar.Append(filemenu, "&File")

        viewmenu = wx.Menu()

        self.viewPropertiesMenuItem = viewmenu.AppendCheckItem(wx.ID_ANY, "&Properties",
                                                               "Toggle the display of the Property Panel")
        self.Bind(wx.EVT_MENU, self.OnViewProps, self.viewPropertiesMenuItem)
        self.viewPropertiesMenuItem.Enable(True)

        self.viewSliderMenuItem = viewmenu.AppendCheckItem(wx.ID_ANY, "&Animation Toolbar",
                                                           "Toggle the display of the Animation Toolbar")
        self.Bind(wx.EVT_MENU, self.OnViewSlider, self.viewSliderMenuItem)
        self.viewSliderMenuItem.Enable(False)

        menubar.Append(viewmenu, "&View")

        self.SetMenuBar(menubar)

    def CreateControls(self):
        # Create a split panel
        splitterstyle = wx.SP_LIVE_UPDATE
        if wx.Platform == "__WXMAC__":
            splitterstyle |= wx.SP_3DSASH
        self.splitPanel = wx.SplitterWindow(self, -1, style=splitterstyle)
        self.splitPanel.Bind(wx.EVT_SIZE, self.OnSplitPanelSize)

        # Create the plot window
        self.plotWindow = PlotWindow(self.splitPanel, -1)
        self.plotWindow.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        self.plotWindow.Bind(EVT_PLOTDATA_CHANGED, self.OnPlotDataChanged)
        self.plotWindow.Bind(EVT_PLOTAXIS_CHANGED, self.OnPlotAxisChanged)
        self.plotWindow.Enable(1)

        # Create the animation toolbar
        self.animationToolbar = AnimationToolbar(self, self.plotWindow)
        self.animationToolbar.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        self.animationToolbar.Bind(EVT_KEYFRAME_CHANGED, self.OnKeyframeChanged)

        # Create the property panel
        # Create a parent wxWindow for the notebook to give a proper background colour for the area behind the tabs
        self.propertyPanel = wx.Window(self.splitPanel, -1)
        # Set the background 'explicitly' to have it stick
        self.propertyPanel.SetBackgroundColour(self.propertyPanel.GetBackgroundColour())
        self.propertyPanel.Bind(wx.EVT_SIZE, self.OnPropertyPanelSize)

        # Create the property notebook
        self.propertyNotebook = wx.Notebook(self.propertyPanel, -1)
        # Don't add size handler for propertyNotebook: re-layout of propertyNotebook doesn't work on Windows

        # Create the content for the property notebook
        self.dataSetPropertyTab = PlotDataSetPanel(self.propertyNotebook, self)
        self.dataSetPropertyTab.Bind(wx.EVT_SIZE, self.OnDataSetPropertyTabSize)
        self.plotPropertyTab = PlotPropertyPanel(self.propertyNotebook, self.plotWindow)
        self.plotPropertyTab.Bind(wx.EVT_SIZE, self.OnPlotPropertyTabSize)

        # Add the content to the notebook
        self.propertyNotebook.AddPage(self.dataSetPropertyTab, "Datasets")
        self.propertyNotebook.AddPage(self.plotPropertyTab, "Plot")

        # Initialize split panel
        self.splitPanel.SetMinimumPaneSize(50)
        self.splitPanel.Initialize(self.plotWindow)

    def CreateLayout(self):
        self.propertyPanelSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.propertyPanelSizer.Add(self.propertyNotebook, 1, wx.EXPAND)
        self.propertyPanel.SetSizerAndFit(self.propertyPanelSizer)

        self.splitPanelSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.splitPanelSizer.Add(self.plotWindow, 1, wx.EXPAND)
        self.splitPanelSizer.Add((self.splitPanel.GetSashSize(), -1), 0, wx.EXPAND)
        self.splitPanelSizer.Add(self.propertyPanel, 0, wx.EXPAND)
        self.splitPanelSizer.Show(1, False)
        self.splitPanelSizer.Show(2, False)
        self.splitPanel.SetSizer(self.splitPanelSizer)

        self.plotWindow.SetMinSize((80, 50))

        self.verticalSizer = wx.BoxSizer(wx.VERTICAL)
        self.verticalSizer.Add(self.splitPanel, 1, wx.EXPAND)
        self.verticalSizer.Add(self.animationToolbar, 0, wx.EXPAND)

        # Hide the animation toolbar by default (so it must be shown explicitly)
        self.verticalSizer.Hide(self.animationToolbar)

        # Hide the property panel by default (so it must be shown explicitly)
        self.propertyPanel.Hide()

        self.SetSizer(self.verticalSizer)
        self.Layout()
        self.UpdateMinSize()

    def UpdateMinSize(self):
        self.SetMinSize(self.GetBestSize())
        currentsize = self.GetSize()
        minsize = self.GetMinSize()
        if currentsize.x < minsize.x or currentsize.y < minsize.y:
            if currentsize.x < minsize.x:
                currentsize.x = minsize.x
            if currentsize.y < minsize.y:
                currentsize.y = minsize.y
            self.SetSize(currentsize)

    def AdjustSplitPanelSashPosition(self):
        # Update sash position in splitter window
        # Try to keep the size of the property pane the same
        sashPosition = -(self.propertyPanel.GetSize()[0] + self.splitPanel.GetSashSize())
        self.splitPanel.SetSashPosition(sashPosition)

    def ShowAnimationToolbar(self, show=True):
        try:
            show = bool(show)
        except ValueError:
            raise TypeError("Show Animation Toolbar parameter should be a boolean (was: '%s')" % str(show))
        self.viewSliderMenuItem.Check(show)

        self.verticalSizer.Show(self.animationToolbar, show)
        self.Layout()

        self.UpdateMinSize()
        self.Refresh()

    def ShowPropertyPanel(self, show=True):
        try:
            show = bool(show)
        except ValueError:
            raise TypeError("Show Property Panel parameter should be a boolean (was: '%s')" % str(show))
        self.viewPropertiesMenuItem.Check(show)

        if show:
            self.splitPanel.SplitVertically(self.plotWindow, self.propertyPanel)
            self.splitPanelSizer.Show(1, True)
            self.splitPanelSizer.Show(2, True)
            self.propertyPanel.Show()
            self.propertyPanel.Fit()
            self.AdjustSplitPanelSashPosition()
        else:
            self.splitPanel.Unsplit(self.propertyPanel)
            self.splitPanelSizer.Show(1, False)
            self.splitPanelSizer.Show(2, False)
        self.splitPanel.Layout()

        self.UpdateMinSize()
        self.Refresh()

    def AddDataSet(self, xdata, ydata):
        xarray = None
        yarray = None
        keyframeSize = None
        keyframeXOffset = None
        keyframeYOffset = None

        # convert ydata to numpy yarray
        if isinstance(ydata, numpy.ndarray):
            yarray = ydata
        else:
            try:
                yarray = numpy.asarray(ydata, dtype=numpy.double)
            except TypeError:
                raise TypeError("plot ydata argument cannot be converted to numpy array. %s" % str(ydata))
        if yarray.ndim == 0:
            # convert scalar to 1D array with one element
            yarray.setshape([1])
        if yarray.ndim > 2:
            raise ValueError("Cannot plot a %d-dimensional numpy array" % yarray.ndim)

        # convert xdata to numpy xarray (if it is available)
        if xdata is not None:
            if isinstance(xdata, numpy.ndarray):
                xarray = xdata
            else:
                try:
                    xarray = numpy.asarray(xdata, dtype=numpy.double)
                except TypeError:
                    raise TypeError("plot xdata argument cannot be converted to numpy array. %s" % str(xdata))
            if xarray.ndim == 0:
                # convert scalar to 1D array with one element
                xarray.setshape([1])
            if xarray.ndim > 2:
                raise ValueError("Cannot plot a %d-dimensional numpy array" % xarray.ndim)

        if yarray.ndim == 2:
            if xarray is not None:
                if xarray.ndim == 2:
                    if xarray.shape[0] != yarray.shape[0] or xarray.shape[1] != yarray.shape[1]:
                        raise ValueError("plot xdata and ydata arguments do not have the same size")
                else:
                    if xarray.shape[0] != yarray.shape[1]:
                        raise ValueError("plot xdata (first dimension) and ydata (second dimension) arguments "
                                         "do not have the same size")
            else:
                xarray = numpy.arange(yarray.shape[1])
        else:
            if xarray is not None and xarray.ndim == 2:
                if xarray.shape[1] != yarray.shape[0]:
                    raise ValueError("plot xdata (second dimension) and ydata (first dimension) arguments "
                                     "do not have the same size")
            elif xarray is None:
                xarray = numpy.arange(yarray.shape[0])

        # create empty attribute/location entry for this data set
        self.dataSetAttributes.extend([dict()])
        self.dataSetLocation.extend([dict()])

        if xarray.ndim == 2 or yarray.ndim == 2:
            self.ShowAnimationToolbar(True)
            if xarray.ndim == 2:
                numDataSets = xarray.shape[0]
            else:
                numDataSets = yarray.shape[0]
            dataSetId = None
            for i in range(numDataSets):
                xdata = xarray[i,:] if xarray.ndim == 2 else xarray
                ydata = yarray[i,:] if yarray.ndim == 2 else yarray
                dataSetId = self.plotWindow.AddDataSet(xdata, ydata, dataSetId=dataSetId)
            self.plotWindow.SetAxisRange(0, *self.plotWindow.GetDataXRange())
            self.plotWindow.SetAxisRange(1, *self.plotWindow.GetDataYRange())
        else:
            dataSetId = self.plotWindow.AddDataSet(xarray, yarray)

        return dataSetId

    def AddDataSetAttribute(self, dataSetId, name, value):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        self.dataSetAttributes[dataSetId][name] = value
        self.dataSetPropertyTab.UpdateAttributes()

    def AddDataSetLocation(self, dataSetId, latitude, longitude):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        if len(self.dataSetLocation[dataSetId]) != 0:
            raise ValueError("dataset already has a location set")
        self.dataSetLocation[dataSetId]['latitude'] = latitude
        self.dataSetLocation[dataSetId]['longitude'] = longitude
        self.dataSetPropertyTab.UpdateLocation()

    def GetNumDataSets(self):
        return self.plotWindow.GetNumDataSets()

    def GetPlotTitle(self):
        return self.plotWindow.GetPlotTitle()

    def SetPlotTitle(self, title):
        try:
            title = str(title)
        except TypeError:
            raise TypeError("parameter 'title' should be a string")
        self.plotWindow.SetPlotTitle(title)
        self.plotPropertyTab.UpdateControls()

    def GetXAxisRange(self):
        return [self.plotWindow.GetAxisRangeMin(0), self.plotWindow.GetAxisRangeMax(0)]

    def SetXAxisRange(self, range):
        try:
            range = tuple(range)
        except TypeError:
            raise TypeError("X Axis range should be a 2-element sequence of numbers (was: '%s')" % str(range))
        try:
            xmin = float(range[0])
            xmax = float(range[1])
        except ValueError:
            raise TypeError("X Axis range should be a 2-element sequence of numbers (was: '%s')" % str(range))
        if xmin > xmax:
            raise ValueError("X Axis range must not denote an empty interval (was: '%s')" % str(range))
        self.plotWindow.SetAxisRange(0, range[0], range[1])
        self.plotPropertyTab.UpdateControls()

    def GetYAxisRange(self):
        return [self.plotWindow.GetAxisRangeMin(1), self.plotWindow.GetAxisRangeMax(1)]

    def SetYAxisRange(self, range):
        try:
            range = tuple(range)
        except TypeError:
            raise TypeError("Y Axis range should be a 2-element sequence of numbers (was: '%s')" % str(range))
        try:
            ymin = float(range[0])
            ymax = float(range[1])
        except ValueError:
            raise TypeError("Y Axis range should be a 2-element sequence of numbers (was: '%s')" % str(range))
        if ymin > ymax:
            raise ValueError("Y Axis range must not denote an empty interval (was: '%s')" % str(range))
        self.plotWindow.SetAxisRange(1, range[0], range[1])
        self.plotPropertyTab.UpdateControls()

    def GetXLogAxis(self):
        return self.plotWindow.GetLogAxis(0)

    def SetXLogAxis(self, log):
        try:
            log = bool(log)
        except ValueError:
            raise TypeError("X Log Axis parameter should be a boolean (was: '%s')" % str(log))
        self.plotWindow.SetLogAxis(0, log)
        self.plotPropertyTab.UpdateControls()

    def GetYLogAxis(self):
        return self.plotWindow.GetLogAxis(1)

    def SetYLogAxis(self, log):
        try:
            log = bool(log)
        except ValueError:
            raise TypeError("Y Log Axis parameter should be a boolean (was: '%s')" % str(log))
        self.plotWindow.SetLogAxis(1, log)
        self.plotPropertyTab.UpdateControls()

    def GetXAxisBase(self):
        return self.plotWindow.GetAxisBase(0)

    def SetXAxisBase(self, base):
        try:
            base = float(base)
        except ValueError:
            raise TypeError("X Axis base should be a float (was: '%s')" % str(base))
        if base < 1:
            raise ValueError("X Axis base should be equal or greater than 1.0 (was: '%s')" % str(base))
        self.plotWindow.SetAxisBase(0, base)
        self.plotPropertyTab.UpdateControls()

    def GetYAxisBase(self):
        return self.plotWindow.GetAxisBase(1)

    def SetYAxisBase(self, base):
        try:
            base = float(base)
        except ValueError:
            raise TypeError("Y Axis base should be a float (was: '%s')" % str(base))
        if base < 1:
            raise ValueError("Y Axis base should be equal or greater than 1.0 (was: '%s')" % str(base))
        self.plotWindow.SetAxisBase(1, base)
        self.plotPropertyTab.UpdateControls()

    def GetXAxisTitle(self):
        return str(self.plotWindow.GetAxisTitle(0))

    def SetXAxisTitle(self, title):
        try:
            title = str(title)
        except TypeError:
            raise ValueError("X Axis title should be a string")
        self.plotWindow.SetAxisTitle(0, title)
        self.plotPropertyTab.UpdateControls()

    def GetYAxisTitle(self):
        return str(self.plotWindow.GetAxisTitle(1))

    def SetYAxisTitle(self, title):
        try:
            title = str(title)
        except TypeError:
            raise ValueError("Y Axis title should be a string")
        self.plotWindow.SetAxisTitle(1, title)
        self.plotPropertyTab.UpdateControls()

    def GetXNumAxisLabels(self):
        return self.plotWindow.GetNumAxisLabels(0)

    def SetXNumAxisLabels(self, numticks):
        try:
            numticks = int(numticks)
        except ValueError:
            raise TypeError("X Axis number of labels should be an integer (was: '%s')" % str(numticks))
        if numticks < 2:
            raise ValueError("X Axis number of labels should be equal or greater than 2 (was: '%s')" % str(numticks))
        self.plotWindow.SetNumAxisLabels(0, numticks)
        self.plotPropertyTab.UpdateControls()

    def GetYNumAxisLabels(self):
        return self.plotWindow.GetNumAxisLabels(1)

    def SetYNumAxisLabels(self, numticks):
        try:
            numticks = int(numticks)
        except ValueError:
            raise TypeError("Y Axis number of labels should be an integer (was: '%s')" % str(numticks))
        if numticks < 2:
            raise ValueError("Y Axis number of labels should be equal or greater than 2 (was: '%s')" % str(numticks))
        self.plotWindow.SetNumAxisLabels(1, numticks)
        self.plotPropertyTab.UpdateControls()

    def GetDataSetLabel(self, dataSetId):
        return str(self.plotWindow.GetDataSetLabel(dataSetId))

    def SetDataSetLabel(self, dataSetId, label):
        try:
            label = str(label)
        except TypeError:
            raise ValueError("DataSet Plot Label should be a string")
        self.plotWindow.SetDataSetLabel(dataSetId, label)
        self.dataSetPropertyTab.UpdateDataSetList()

    def GetPlotLines(self, dataSetId):
        return self.plotWindow.GetPlotLines(dataSetId)

    def SetPlotLines(self, dataSetId, plotLines):
        try:
            plotLines = bool(plotLines)
        except ValueError:
            raise TypeError("PlotLines parameter should be a boolean (was: '%s')" % str(plotLines))
        self.plotWindow.SetPlotLines(dataSetId, plotLines)

    def GetLineWidth(self, dataSetId):
        return self.plotWindow.GetLineWidth(dataSetId)

    def SetLineWidth(self, dataSetId, lineWidth):
        try:
            lineWidth = float(lineWidth)
        except ValueError:
            raise TypeError("Line Width should be a float (was: '%s')" % str(lineWidth))
        if lineWidth < 0:
            raise ValueError("Line Width should be equal or greater than 0.0 (was: '%s')" % str(lineWidth))
        self.plotWindow.SetLineWidth(dataSetId, lineWidth)

    def GetLineStipplePattern(self, dataSetId):
        return self.plotWindow.GetLineStipplePattern(dataSetId)

    def SetLineStipplePattern(self, dataSetId, lineStipplePattern):
        try:
            lineStipplePattern = int(lineStipplePattern)
        except ValueError:
            raise TypeError("Line Stipple Pattern should be an (hex) integer (was: '%s')" % str(lineStipplePattern))
        if lineStipplePattern <= 0 or lineStipplePattern > 65535:
            raise ValueError("Line Stipple Pattern should be greater than 0 and less than 65536 (was: '%s')" %
                             str(lineStipplePattern))
        self.plotWindow.SetLineStipplePattern(dataSetId, lineStipplePattern)

    def GetPlotPoints(self, dataSetId):
        return self.plotWindow.GetPlotPoints(dataSetId)

    def SetPlotPoints(self, dataSetId, plotPoints):
        try:
            plotPoints = bool(plotPoints)
        except ValueError:
            raise TypeError("PlotPoints parameter should be a boolean (was: '%s')" % str(plotPoints))
        self.plotWindow.SetPlotPoints(dataSetId, plotPoints)

    def GetPointSize(self, dataSetId):
        return self.plotWindow.GetPointSize(dataSetId)

    def SetPointSize(self, dataSetId, pointSize):
        try:
            pointSize = float(pointSize)
        except ValueError:
            raise TypeError("Point Size should be a float (was: '%s')" % str(pointSize))
        if pointSize < 0:
            raise ValueError("Point Size should be equal or greater than 0.0 (was: '%s')" % str(pointSize))
        self.plotWindow.SetPointSize(dataSetId, pointSize)

    def GetPlotColor(self, dataSetId):
        return self.plotWindow.GetPlotColor(dataSetId)

    def SetPlotColor(self, dataSetId, color):
        try:
            color = tuple(color)
        except TypeError:
            raise TypeError("Plot Color should be a 3-element sequence of numbers (was: '%s')" % str(color))
        try:
            r = float(color[0])
            g = float(color[1])
            b = float(color[2])
        except ValueError:
            raise TypeError("Plot Color should be a 3-element sequence of numbers (was: '%s')" % str(color))
        if r < 0 or g < 0 or b < 0 or r > 1 or g > 1 or b > 1:
            raise ValueError("Plot Color must contain numbers between 0.0 and 1.0 (was: '%s')" % str(color))
        self.plotWindow.SetPlotColor(dataSetId, color)

    def GetOpacity(self, dataSetId):
        return self.plotWindow.GetOpacity(dataSetId)

    def SetOpacity(self, dataSetId, opacity):
        try:
            opacity = float(opacity)
        except ValueError:
            raise TypeError("Opacity should be a float (was: '%s')" % str(opacity))
        if opacity < 0 or opacity > 1:
            raise ValueError("Opacity should be between 0.0 and 1.0 (was: '%s')" % str(opacity))
        self.plotWindow.SetOpacity(dataSetId, opacity)

    def GetNumKeyframes(self):
        return self.plotWindow.GetNumKeyframes()

    def GetKeyframe(self):
        return self.plotWindow.GetKeyframe()

    def SetKeyframe(self, keyframe):
        self.animationToolbar.UpdateNumKeyframes()
        self.animationToolbar.SetKeyframe(keyframe)

    def ExportToImageFile(self, filename, imageType):
        self.plotWindow.ExportToImageFile(filename, imageType)

    def OnClose(self, event):
        # Make sure this wxFrame is not closing down
        if self.closingDown:
            return

        # Fix for EVT_KILL_FOCUS crashes
        win = wx.Window.FindFocus()
        if win is not None:
            win.Disconnect(-1, -1, wx.wxEVT_KILL_FOCUS)

        self.closingDown = True
        self.animationToolbar.Destroy()
        self.Destroy()

    def OnKeyDown(self, event):
        # Check if space was pressed
        if (event.GetKeyCode() == 32):
            self.animationToolbar.TogglePlayPause()
        event.Skip()

    def OnPlotAxisChanged(self, event):
        self.plotPropertyTab.UpdateAxes()

    def OnPlotDataChanged(self, event):
        self.dataSetPropertyTab.UpdateDataSetList()
        self.animationToolbar.UpdateNumKeyframes()
        self.viewSliderMenuItem.Enable(self.plotWindow.GetNumKeyframes() > 1)

    def OnKeyframeChanged(self, event):
        # Make sure this wxFrame is not closing down
        if self.closingDown:
            return

        self.plotWindow.SetKeyframe(event.keyframe)
        self.dataSetPropertyTab.UpdateAttributes()
        self.dataSetPropertyTab.UpdateLocation()

    def OnSplitPanelSize(self, event):
        if self.splitPanel.IsSplit():
            self.AdjustSplitPanelSashPosition()
        event.Skip()

    def OnPropertyPanelSize(self, event):
        self.propertyPanel.Layout()
        event.Skip()

    def OnDataSetPropertyTabSize(self, event):
        self.dataSetPropertyTab.Layout()
        event.Skip()

    def OnPlotPropertyTabSize(self, event):
        self.plotPropertyTab.Layout()
        event.Skip()

    def OnViewSlider(self, event):
        self.ShowAnimationToolbar(event.IsChecked())

    def OnViewProps(self, event):
        self.ShowPropertyPanel(event.IsChecked())

    def OnSave(self, event):
        imagetypeinfo = [("TIFF", "tif"), ("Windows Bitmap", "bmp"), ("JPEG", "jpg"), ("PNG", "png"), ("PNM", "pnm")]
        dialog = TypedSaveFileDialog(self, title="Save Image",
                                     initialdir=str(wx.Config.Get().Read('DirectoryLocation/Export')),
                                     typeinfo=imagetypeinfo)
        if dialog.ShowModal() == wx.ID_OK:
            wx.Config.Get().Write('DirectoryLocation/Export', os.path.dirname(dialog.filename))
            self.ExportToImageFile(dialog.filename, dialog.ext)



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy
import wx
import wx.lib.newevent

from .util import DetermineCharSize

ValueChangedEvent, EVT_VALUE_CHANGED = wx.lib.newevent.NewCommandEvent()


class ColorBar(wx.Panel):

    def __init__(self, parent, showTicks=False):
        wx.Panel.__init__(self, parent, -1)

        self.colorTable = None
        self.showTicks = showTicks
        self.selectedTick = -1
        self.observertag = -1

        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SIZE, self.OnSize)

        self.UpdateBitmap()

    def SetColorTable(self, colorTable):
        def onColorTableChanged(caller, event, calldata=None):
            self.UpdateBitmap()
        if self.colorTable is not None:
            self.colorTable.RemoveObserver(self.observertag)
        self.colorTable = colorTable
        if self.colorTable is not None:
            self.observertag = self.colorTable.AddObserver("ColorTableChanged", onColorTableChanged)
        self.UpdateBitmap()

    def SelectTick(self, tickId):
        if tickId != self.selectedTick:
            self.selectedTick = tickId
            if self.showTicks:
                self.UpdateBitmap()

    def ShowTicks(self, show=True):
        if self.showTicks != show:
            self.showTicks = show
            self.UpdateBitmap()

    def UpdateBitmap(self):
        width, height = self.GetSize()
        width = max(width, 1)
        height = max(height, 1)

        arr = numpy.empty(shape=(height, width, 4), dtype='u1')
        if arr.size == 0:
            return
        arr[:] = 0

        if self.colorTable is not None:
            for x in range(width):
                index = (x * self.colorTable.GetNumTableColors()) / width
                r, g, b, a = self.colorTable.GetTableColor(index)
                arr[:, x, 0] = r * 255
                arr[:, x, 1] = g * 255
                arr[:, x, 2] = b * 255
                arr[:, x, 3] = a * 255

            tickHeight = 10
            if self.showTicks and height > tickHeight:
                arr[height - tickHeight:height, :, :] = 0
                for index in range(self.colorTable.GetNumGradientEdges()):
                    tickPos = int(self.colorTable.GetGradientEdgeValue(index)[0] * (width - 1))
                    arr[height - tickHeight + 2:height, tickPos, 3] = 255
                    if index == self.selectedTick:
                        if tickPos > 0:
                            arr[height - 1, tickPos - 4:tickPos, 3] = 255
                            arr[height - 2, tickPos - 3:tickPos, 3] = 255
                            arr[height - 3, tickPos - 2:tickPos, 3] = 255
                            arr[height - 4, tickPos - 1, 3] = 255
                        if tickPos < width - 1:
                            arr[height - 1, tickPos + 1:tickPos + 5, 3] = 255
                            arr[height - 2, tickPos + 1:tickPos + 4, 3] = 255
                            arr[height - 3, tickPos + 1:tickPos + 3, 3] = 255
                            arr[height - 4, tickPos + 1, 3] = 255

        self.bitmap = wx.Bitmap.FromBufferRGBA(width, height, arr)
        self.Refresh()

    def OnSize(self, evt):
        self.UpdateBitmap()

    def OnPaint(self, evt):
        dc = wx.PaintDC(self)
        dc.DrawBitmap(self.bitmap, 0, 0, True)


class ColorBox(wx.Panel):

    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SIZE, self.OnSize)

        self.color = [0.0, 0.0, 0.0, 1.0]
        self.UpdateBitmap()

    def SetColor(self, color):
        self.color = color
        self.UpdateBitmap()

    def UpdateBitmap(self):
        width, height = self.GetSize()
        width = max(width, 1)
        height = max(height, 1)

        arr = numpy.empty(shape=(height, width, 4), dtype='u1')
        arr[:, :, 0] = self.color[0] * 255
        arr[:, :, 1] = self.color[1] * 255
        arr[:, :, 2] = self.color[2] * 255
        arr[:, :, 3] = self.color[3] * 255
        self.bitmap = wx.Bitmap.FromBufferRGBA(width, height, arr)
        self.Refresh()

    def OnSize(self, evt):
        self.UpdateBitmap()

    def OnPaint(self, evt):
        dc = wx.PaintDC(self)
        dc.DrawBitmap(self.bitmap, 0, 0, True)


class ValueSlider(wx.Panel):

    def __init__(self, parent, title, minValue=0, maxValue=255):
        wx.Panel.__init__(self, parent, -1)

        self.parent = parent
        self.hasfocus = False
        self.title = title
        self.minValue = int(minValue)
        self.maxValue = int(maxValue)

        # Create and configure all widgets
        self.CreateControls()
        self.CreateLayout()

    def CreateControls(self):
        (charWidth, charHeight) = DetermineCharSize(self)

        self.label = wx.StaticText(self, -1, "%s:" % self.title)

        self.slider = wx.Slider(self, -1, value=0, minValue=self.minValue, maxValue=self.maxValue, size=(120, -1))
        self.slider.Bind(wx.EVT_SLIDER, self.OnSlider)

        self.text = wx.TextCtrl(self, -1, "0", style=wx.TE_PROCESS_ENTER | wx.TE_RIGHT, size=(4 * charWidth, -1))
        self.text.Bind(wx.EVT_TEXT_ENTER, self.OnText)
        self.text.Bind(wx.EVT_SET_FOCUS, self.OnSetFocus)
        self.text.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)

    def CreateLayout(self):
        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer.Add(self.label, 0, wx.ALIGN_CENTRE | wx.RIGHT, border=3)
        self.sizer.Add(self.slider, 1, wx.ALIGN_CENTRE | wx.RIGHT, border=5)
        self.sizer.Add(self.text, 0, wx.ALIGN_CENTRE)
        self.SetSizer(self.sizer)

    def GetValue(self):
        assert str(self.slider.GetValue()) == self.text.GetValue()
        return self.slider.GetValue()

    def SetValue(self, value):
        self.slider.SetValue(int(value))
        self.text.SetValue(str(int(value)))

    def SetRange(self, minValue, maxValue):
        minValue = int(minValue)
        maxValue = int(maxValue)

        if minValue != self.minValue or maxValue != self.maxValue:
            self.minValue = minValue
            self.maxValue = maxValue
            self.slider.SetRange(self.minValue, self.maxValue)
            self.text.SetValue(str(self.slider.GetValue()))

    def OnSlider(self, event):
        self.text.SetValue(str(self.slider.GetValue()))
        wx.PostEvent(self, ValueChangedEvent(self.GetId()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()

    def OnText(self, event):
        self.slider.SetValue(max(min(int(self.text.GetValue()), self.maxValue), self.minValue))
        wx.PostEvent(self, ValueChangedEvent(self.GetId()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()

    def OnSetFocus(self, event):
        self.hasfocus = True
        self.text.SetSelection(-1, -1)
        event.Skip()

    def OnKillFocus(self, event):
        self.slider.SetValue(max(min(int(self.text.GetValue()), self.maxValue), self.minValue))
        if self.hasfocus:
            self.text.SetSelection(0, 0)
            self.hasfocus = False
        wx.PostEvent(self, ValueChangedEvent(self.GetId()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()
        event.Skip()


class ColorTablePanel(wx.Panel):

    def __init__(self, parent, plotWindow):
        panelstyle = wx.TAB_TRAVERSAL
        if wx.Platform == '__WXGTK__':
            panelstyle |= wx.SUNKEN_BORDER
        wx.Panel.__init__(self, parent, -1, style=panelstyle)

        # Set initial state
        self.liveUpdate = True
        self.colorTable = None
        self.plotWindow = plotWindow
        self.xSliderScale = 1000
        self.currentIndex = 0
        self.observertag = -1

        # Create and configure all widgets
        self.CreateControls()
        self.CreateLayout()

        self.UpdateAll()

    def CreateControls(self):
        (charWidth, charHeight) = DetermineCharSize(self)

        self.namedColorTableChoice = wx.Choice(self, -1, size=(10 * charWidth, -1),
                                               choices=["", "Default", "BlackToWhite", "WhiteToBlack", "GreenToRed",
                                                        "RedToGreen", "Cloud", "Rainbow", "Ozone", "Blackbody",
                                                        "Aerosol"])
        self.namedColorTableChoice.Bind(wx.EVT_CHOICE, self.OnNamedColorTabelChoice)

        self.colorBar = ColorBar(self, showTicks=True)
        self.colorBar.SetSize((100, 40))
        self.colorBar.SelectTick(0)

        self.pointLabel = wx.StaticText(self, -1, "Gradient Edge Point:")

        self.pointChoice = wx.Choice(self, -1, name="Gradient Edge Point:", size=(3 * charWidth, -1))
        self.pointChoice.Bind(wx.EVT_CHOICE, self.OnEdgePointChanged)

        self.xSlider = ValueSlider(self, 'X', 0, self.xSliderScale)
        self.xSlider.Bind(EVT_VALUE_CHANGED, self.OnValueChanged)

        self.rSlider = ValueSlider(self, 'R')
        self.rSlider.Bind(EVT_VALUE_CHANGED, self.OnValueChanged)

        self.gSlider = ValueSlider(self, 'G')
        self.gSlider.Bind(EVT_VALUE_CHANGED, self.OnValueChanged)

        self.bSlider = ValueSlider(self, 'B')
        self.bSlider.Bind(EVT_VALUE_CHANGED, self.OnValueChanged)

        self.aSlider = ValueSlider(self, 'A')
        self.aSlider.Bind(EVT_VALUE_CHANGED, self.OnValueChanged)
        self.aSlider.SetValue(255)

        self.colorBox = ColorBox(self)
        self.colorBox.SetSize((50, 50))
        self.colorBox.SetColor(self.GetColor())

        self.insertButton = wx.Button(self, -1, "Add Point")
        self.Bind(wx.EVT_BUTTON, self.OnInsert)

        self.removeButton = wx.Button(self, -1, "Remove Point")
        self.removeButton.Bind(wx.EVT_BUTTON, self.OnRemove)

        self.liveUpdateCtrl = wx.CheckBox(self, -1, "Live update")
        self.liveUpdateCtrl.SetToolTip(wx.ToolTip("If disabled, you will only see color table changes in the plot "
                                                  "area once you interact with the plot area (e.g. click in it with "
                                                  "the mouse). Since updating the plot area can be time consuming, "
                                                  "disabling this option allows you to quickly edit a color table"))
        self.liveUpdateCtrl.Bind(wx.EVT_CHECKBOX, self.OnLiveUpdate)
        self.liveUpdateCtrl.SetValue(True)

    def CreateLayout(self):
        butsizer = wx.BoxSizer(wx.HORIZONTAL)
        butsizer.Add(self.insertButton, 0, wx.ALL, border=5)
        butsizer.Add(self.removeButton, 0, wx.ALL, border=5)

        colorslidersizer = wx.BoxSizer(wx.VERTICAL)
        colorslidersizer.Add(self.xSlider, 1, wx.EXPAND | wx.BOTTOM, border=10)
        colorslidersizer.Add(self.rSlider, 1, wx.EXPAND | wx.BOTTOM, border=5)
        colorslidersizer.Add(self.gSlider, 1, wx.EXPAND | wx.BOTTOM, border=5)
        colorslidersizer.Add(self.bSlider, 1, wx.EXPAND | wx.BOTTOM, border=10)
        colorslidersizer.Add(self.aSlider, 1, wx.EXPAND)

        colorsizer = wx.BoxSizer(wx.HORIZONTAL)
        colorsizer.Add(colorslidersizer, 1, wx.EXPAND | wx.ALIGN_LEFT)
        colorsizer.Add((10, 0), 0)
        colorsizer.Add(self.colorBox, 0, wx.ALIGN_LEFT)

        pointchoicesizer = wx.BoxSizer(wx.HORIZONTAL)
        pointchoicesizer.Add(self.pointLabel, 0, wx.ALIGN_LEFT)
        pointchoicesizer.Add(self.pointChoice, 1, wx.EXPAND | wx.ALIGN_LEFT | wx.LEFT, border=10)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.namedColorTableChoice, 0, wx.EXPAND | wx.ALL, border=5)
        sizer.Add(self.colorBar, 0, wx.EXPAND | wx.ALL, border=5)
        sizer.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND | wx.TOP | wx.BOTTOM, border=5)
        sizer.Add(pointchoicesizer, 0, wx.EXPAND | wx.ALIGN_LEFT | wx.LEFT | wx.RIGHT, border=5)
        sizer.Add(butsizer, 0, wx.ALIGN_RIGHT)
        sizer.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND | wx.TOP | wx.BOTTOM, border=5)
        sizer.Add(colorsizer, 0, wx.EXPAND | wx.ALL, border=5)
        sizer.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND | wx.TOP | wx.BOTTOM, border=5)
        sizer.Add(self.liveUpdateCtrl, 0, wx.ALIGN_RIGHT | wx.TOP | wx.RIGHT, border=5)

        self.SetSizer(sizer)

    def SetColorTable(self, colorTable):
        def onColorTableChanged(caller, event, calldata=None):
            self.UpdateAll()
            if self.liveUpdate:
                self.plotWindow.Refresh()
        if self.colorTable is not None:
            self.colorTable.RemoveObserver(self.observertag)
        self.colorTable = colorTable
        if self.colorTable is not None:
            self.observertag = self.colorTable.AddObserver("ColorTableChanged", onColorTableChanged)
        self.UpdateAll()

    def GetColor(self):
        r = float(self.rSlider.GetValue()) / 255.
        g = float(self.gSlider.GetValue()) / 255.
        b = float(self.bSlider.GetValue()) / 255.
        a = float(self.aSlider.GetValue()) / 255.
        return (r, g, b, a)

    def SetColor(self, color):
        self.rSlider.SetValue(int(color[0] * 255))
        self.gSlider.SetValue(int(color[1] * 255))
        self.bSlider.SetValue(int(color[2] * 255))
        self.aSlider.SetValue(int(color[3] * 255))
        self.colorBox.SetColor(color)

    def GetEdgePosition(self):
        return float(self.xSlider.GetValue()) / self.xSliderScale

    def SetEdgePosition(self, x):
        self.xSlider.SetValue(x * self.xSliderScale)

    def OnNamedColorTabelChoice(self, event):
        name = self.namedColorTableChoice.GetStringSelection()
        if name != "":
            self.colorTable.SetColorTableByName(name)

    def OnValueChanged(self, event):
        r, g, b, a = self.GetColor()
        x = self.GetEdgePosition()
        self.colorTable.SetGradientEdgeValue(self.currentIndex, x, r, g, b, a)

    def OnEdgePointChanged(self, event):
        self.currentIndex = self.pointChoice.GetSelection()
        self.colorBar.SelectTick(self.currentIndex)
        self.UpdateEdgePoint()

    def OnInsert(self, event):
        r, g, b, a = self.GetColor()
        x = self.GetEdgePosition()
        if self.currentIndex == 0:
            x = (x + self.colorTable.GetGradientEdgeValue(self.currentIndex + 1)[0]) / 2.0
        else:
            x = (x + self.colorTable.GetGradientEdgeValue(self.currentIndex - 1)[0]) / 2.0
        newIndex = self.colorTable.InsertGradientEdgeValue(x, r, g, b, a)
        # A refresh will automatically happen via the OnColorTableChanged event handler
        if newIndex != self.currentIndex:
            self.currentIndex = newIndex
            self.colorBar.SelectTick(self.currentIndex)
            self.UpdateEdgePoint()

    def OnRemove(self, event):
        self.colorTable.RemoveGradientEdgeValue(self.currentIndex)
        # A refresh will automatically happen via the OnColorTableChanged event handler

    def OnLiveUpdate(self, event):
        self.liveUpdate = self.liveUpdateCtrl.IsChecked()
        if self.liveUpdate:
            self.plotWindow.Refresh()

    def UpdateEdgePoint(self):
        x, r, g, b, a = self.colorTable.GetGradientEdgeValue(self.currentIndex)
        sliderMin = 0
        if self.currentIndex > 0:
            sliderMin = self.colorTable.GetGradientEdgeValue(self.currentIndex - 1)[0] + 1.0 / self.xSliderScale
        sliderMax = 1
        if self.currentIndex < self.colorTable.GetNumGradientEdges() - 1:
            sliderMax = self.colorTable.GetGradientEdgeValue(self.currentIndex + 1)[0] - 1.0 / self.xSliderScale
        self.xSlider.SetRange(sliderMin * self.xSliderScale, sliderMax * self.xSliderScale)
        self.SetColor((r, g, b, a))
        self.SetEdgePosition(x)

        modifiableEdge = self.currentIndex > 0 and self.currentIndex < self.colorTable.GetNumGradientEdges() - 1
        self.xSlider.Enable(modifiableEdge)
        self.removeButton.Enable(modifiableEdge)

    def UpdateAll(self):
        if self.colorTable is not None:
            name = str(self.colorTable.GetColorTableName())
            if name is None:
                self.namedColorTableChoice.SetSelection(0)
            else:
                self.namedColorTableChoice.SetStringSelection(name)
            numEdges = self.colorTable.GetNumGradientEdges()
            if self.currentIndex >= numEdges:
                self.currentIndex = numEdges - 1
            self.pointChoice.SetItems([str(i) for i in range(numEdges)])
            self.pointChoice.Select(self.currentIndex)
            self.colorBar.SelectTick(self.currentIndex)
            self.UpdateEdgePoint()
        else:
            self.namedColorTableChoice.SetSelection(0)
            self.namedColorTableChoice.Disable()
            self.colorBar.SelectTick(-1)
        self.namedColorTableChoice.Enable(self.colorTable is not None)
        self.pointChoice.Enable(self.colorTable is not None)
        self.insertButton.Enable(self.colorTable is not None)
        self.removeButton.Enable(self.colorTable is not None)
        self.xSlider.Enable(self.colorTable is not None)
        self.rSlider.Enable(self.colorTable is not None)
        self.gSlider.Enable(self.colorTable is not None)
        self.bSlider.Enable(self.colorTable is not None)
        self.aSlider.Enable(self.colorTable is not None)



# This is an example VISAN script for the GOME_L1 product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing GOME L1 products (calibrated and converted using the GOME Lvl1 extractor).

# This example will then take the first product it finds in this directory and
# for that product plot the measured spectra


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all files in productdir ending with '.el1'
    files = glob.glob(os.path.join(productdir, "*.el1"))
    if len(files) == 0:
        print(("Could not find any GOME_L1_EXTRACTED files in directory '" + productdir + "'"))
        return

    # We only ingest the first file from the list
    product = harp.import_product(files[0])

    plot(product, ymin=0, ymax=65535, name=os.path.basename(files[0]), title="GOME_L1 spectral readout example")


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import wx

from .axispropertypanel import AxisPropertyPanel
from .labeledtextctrl import LabeledTextCtrl, EVT_VALUE_CHANGED
from .plotwindow import X_AXIS, Y_AXIS
from .util import DetermineCharSize


class PlotPropertyPanel(wx.Panel):

    def __init__(self, parent, plotWindow):
        windowstyle = wx.TAB_TRAVERSAL
        if wx.Platform == '__WXGTK__':
            windowstyle |= wx.SUNKEN_BORDER
        wx.Panel.__init__(self, parent, -1, style=windowstyle)

        # Set initial state
        self.plotFrame = parent
        self.plotWindow = plotWindow

        # Create and configure all widgets
        self.CreateControls()
        self.CreateLayout()

    def CreateControls(self):
        (charWidth, charHeight) = DetermineCharSize(self)

        self.titleCtrl = LabeledTextCtrl(self, -1, label="Plot Title:", value=self.plotWindow.GetPlotTitle(),
                                         size=(10 * charWidth, -1), style=wx.TE_PROCESS_ENTER)
        self.titleCtrl.SetToolTip(wx.ToolTip("The plot title will be displayed centered at the top of the plot."))
        self.titleCtrl.Bind(EVT_VALUE_CHANGED, self.OnTitle)

        self.axisNotebook = wx.Notebook(self, -1)

        self.xAxisPanel = AxisPropertyPanel(self.axisNotebook, self.plotWindow, X_AXIS)
        self.axisNotebook.AddPage(self.xAxisPanel, "X-Axis")

        self.yAxisPanel = AxisPropertyPanel(self.axisNotebook, self.plotWindow, Y_AXIS)
        self.axisNotebook.AddPage(self.yAxisPanel, "Y-Axis")

    def CreateLayout(self):
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.titleCtrl, 0, wx.EXPAND | wx.ALL, border=10)
        sizer.Add(self.axisNotebook, 1, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.BOTTOM, border=2)

        self.SetSizerAndFit(sizer)

    def UpdateControls(self):
        self.titleCtrl.SetValue(self.plotWindow.GetPlotTitle())
        self.xAxisPanel.UpdateControls()
        self.yAxisPanel.UpdateControls()

    def UpdateAxes(self):
        self.xAxisPanel.UpdateRangeFromPlot()
        self.yAxisPanel.UpdateRangeFromPlot()

    def OnTitle(self, event):
        self.plotWindow.SetPlotTitle(event.value)



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os

import wx
import coda


class PreferencesDialog(wx.Dialog):

    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, -1, "Preferences")
        self.config = wx.Config.Get()

        self.currentdir = self.config.Read('DirectoryLocation/Products')

        self.CreateControls()
        self.CreateLayout()
        self.InstallToolTips()
        self.InstallEventListeners()

    def CreateControls(self):
        self.locLabel = wx.StaticText(self, -1, "&Location of your product-file directory:",
                                      size=wx.DLG_UNIT(self, -1, -1), style=wx.ALIGN_RIGHT)
        self.locField = wx.TextCtrl(self, -1, size=wx.DLG_UNIT(self, 150, -1), style=wx.TE_PROCESS_ENTER)
        self.browseButton = wx.Button(self, -1, "Browse...")

        self.filterRecordFields = wx.CheckBox(self, -1, "Show hidden and unavailable record fields")
        self.filterRecordFields.SetValue(not coda.get_option_filter_record_fields())

        self.performConversions = wx.CheckBox(self, -1, "Perform conversions")
        self.performConversions.SetValue(coda.get_option_perform_conversions())

        self.showByteSize = wx.CheckBox(self, -1, "Show byte sizes")
        self.showByteSize.SetValue(self.config.ReadBool('ProductBrowser/ShowByteSize'))

        self.cancelButton = wx.Button(self, wx.ID_CANCEL, "Cancel")
        self.saveButton = wx.Button(self, wx.ID_OK, "Save")

        self.saveButton.Enable(len(self.currentdir) > 0)

        locdefault = self.currentdir
        if locdefault != "" and locdefault[-1] != os.sep:
            locdefault += os.sep

        self.locField.SetValue(locdefault)
        self.locField.SetInsertionPointEnd()
        self.SetDefaultItem(self.locField)
        self.locField.SetFocus()

    def CreateLayout(self):
        hsizer1 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer1.Add(self.locField, 1, wx.EXPAND | wx.RIGHT, border=10)
        hsizer1.Add(self.browseButton, 0, wx.EXPAND)

        hsizer2 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer2.Add((0, 0), 1, wx.EXPAND)
        hsizer2.Add(self.cancelButton, 0, wx.EXPAND | wx.RIGHT, border=10)
        hsizer2.Add(self.saveButton, 0, wx.EXPAND)

        vsizer = wx.BoxSizer(wx.VERTICAL)
        vsizer.Add(self.locLabel, 0, wx.LEFT | wx.TOP | wx.RIGHT, border=10)
        vsizer.Add(hsizer1, 0, wx.EXPAND | wx.ALL, border=10)
        vsizer.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND | wx.LEFT | wx.BOTTOM | wx.RIGHT, 10)
        vsizer.Add(self.filterRecordFields, 0, wx.LEFT | wx.BOTTOM | wx.RIGHT, 10)
        vsizer.Add(self.performConversions, 0, wx.LEFT | wx.BOTTOM | wx.RIGHT, 10)
        vsizer.Add(self.showByteSize, 0, wx.LEFT | wx.BOTTOM | wx.RIGHT, 10)
        vsizer.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND | wx.LEFT | wx.BOTTOM | wx.RIGHT, 10)
        vsizer.Add(hsizer2, 0, wx.EXPAND | wx.LEFT | wx.BOTTOM | wx.RIGHT, border=10)
        vsizer.Fit(self)

        self.SetAutoLayout(True)
        self.SetSizerAndFit(vsizer)
        self.CentreOnParent()

    def InstallToolTips(self):
        self.locField.SetToolTip(wx.ToolTip("The path to the directory where product files are located."))
        self.browseButton.SetToolTip(wx.ToolTip("Open a path selector dialog box."))
        self.filterRecordFields.SetToolTip(wx.ToolTip("If checked, hidden and unavailable record fields "
                                                      "will be shown in the product browser."))
        self.performConversions.SetToolTip(wx.ToolTip("If checked, units and values are converted as specified "
                                                      "in the data dictionary."))
        self.showByteSize.SetToolTip(wx.ToolTip("If checked, byte sizes will be shown in the product browser. "
                                                "Determining byte sizes may take much time, so the default is not "
                                                "to show them."))

    def InstallEventListeners(self):
        self.cancelButton.Bind(wx.EVT_BUTTON, self.OnCancel)
        self.saveButton.Bind(wx.EVT_BUTTON, self.OnSave)
        self.browseButton.Bind(wx.EVT_BUTTON, self.OnBrowse)
        self.locField.Bind(wx.EVT_TEXT_ENTER, self.OnLocFieldEnter)
        self.locField.Bind(wx.EVT_TEXT, self.OnLocFieldChar)
        self.Bind(wx.EVT_CLOSE, self.OnClose)

    def OnClose(self, event):
        self.EndModal(wx.ID_CANCEL)

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

    def OnSave(self, event):
        self.pathname = self.locField.GetValue()

        self.pathname = os.path.abspath(os.path.expanduser(os.path.expandvars(self.pathname)))

        if not os.path.isdir(self.pathname):
            wx.MessageBox(parent=self, message="Product-file directory location is not a valid directory.",
                          caption="Error", style=wx.OK | wx.ICON_ERROR | wx.STAY_ON_TOP)
            return

        coda.set_option_filter_record_fields(not self.filterRecordFields.GetValue())
        coda.set_option_perform_conversions(self.performConversions.GetValue())

        self.config.Write('DirectoryLocation/Products', self.pathname)
        self.config.WriteBool('ProductBrowser/ShowByteSize', self.showByteSize.GetValue())
        self.config.WriteBool('CODA/PerformConversions', coda.get_option_perform_conversions())
        self.config.WriteBool('CODA/FilterRecordFields', coda.get_option_filter_record_fields())
        self.config.Flush()

        self.EndModal(wx.ID_OK)

    def OnLocFieldChar(self, event):
        self.saveButton.Enable(self.locField.GetValue() != "")

    def OnLocFieldEnter(self, event):
        self.OnSave(None)

    def OnBrowse(self, event):
        directory = os.path.dirname(self.locField.GetValue())
        if not directory:
            directory = self.currentdir

        directory = os.path.abspath(os.path.expanduser(os.path.expandvars(directory)))

        pathname = wx.DirSelector("Select Path", directory, wx.FD_SAVE, parent=self)
        if pathname:
            self.locField.SetValue(pathname)
            self.locField.SetInsertionPointEnd()
            self.saveButton.SetFocus()
            self.currentdir = os.path.dirname(pathname)



# This is an example VISAN script for the ALD_U_N_2A product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing ALD_U_N_2A products.

# This example will then take the first product it finds in this directory and
# for this product plot the Mie and Rayleigh wind profiles for measurements and observations


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all DBL files in productdir containing with 'ALD_U_N_2A'
    files = glob.glob(os.path.join(productdir, "*ALD_U_N_2A*.DBL"))
    if len(files) == 0:
        print(("Could not find any ALD_U_N_2A files in directory '" + productdir + "'"))
        return

    # We only take the first file
    filename = files[0]

    # We turn the lat/lon values per profile point to single lat/lon values per profile
    # This allows us to show the location of each profile in the plot property panel
    operations = "derive(latitude {time});derive(longitude {time})"
    # The validity flag is the same for each quantity, so we just use the one
    # for the extinction coefficient to filter out the levels where no
    # quantity is available (i.e. where the validity flag equals 0)
    operations += ";extinction_coefficient_validity!=0"

    data = harp.import_product(filename, operations)
    plot(data, title="ALD_U_N_2A extinction coefficient")
    plot(data, title="ALD_U_N_2A backscatter coefficient", value="backscatter_coefficient")
    plot(data, title="ALD_U_N_2A optical depth", value="optical_depth")


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import wx
import wx.lib.newevent

ValueChangedEvent, EVT_VALUE_CHANGED = wx.lib.newevent.NewCommandEvent()


class LabeledTextCtrl(wx.Panel):

    def __init__(self, parent=None, id=-1, label=None, value=None, formatstring=None, **keywd):
        wx.Panel.__init__(self, parent, id)

        self.formatstring = formatstring

        if label:
            self.label = wx.StaticText(self, -1, label, size=(-1, -1))

        if value is None:
            value = ""
        self.text = wx.TextCtrl(self, -1, value=str(value), **keywd)
        self.text.Bind(wx.EVT_TEXT_ENTER, self.OnTextEnter)
        self.text.Bind(wx.EVT_SET_FOCUS, self.OnSetFocus)
        self.text.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)

        self.hasfocus = False

        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        if label:
            self.sizer.Add(self.label, 0, wx.ALIGN_CENTER | wx.RIGHT, border=3)
        self.sizer.Add(self.text, 1, wx.ALIGN_CENTER | wx.RIGHT)

        self.SetAutoLayout(True)
        self.SetSizerAndFit(self.sizer)

    def GetValue(self):
        return self.text.GetValue()

    def SetValue(self, value):
        if value is None:
            value = ""
        if self.formatstring:
            try:
                return self.text.SetValue(self.formatstring % value)
            except TypeError:
                raise RuntimeError("Type of '%s' does not match format string '%s'" % (value, self.formatstring))
        else:
            return self.text.SetValue(str(value))

    def SetFormat(self, formatstring):
        self.formatstring = formatstring

    def OnTextEnter(self, event):
        self.text.SetSelection(-1, -1)
        wx.PostEvent(self, ValueChangedEvent(self.GetId(), value=event.GetString()))
        if wx.Platform == '__WXMAC__':
            wx.Yield()

    def OnSetFocus(self, event):
        self.hasfocus = True
        self.text.SetSelection(-1, -1)
        event.Skip()

    def OnKillFocus(self, event):
        event.Skip()
        self.text.SetSelection(0, 0)
        if self.text.IsModified() and self.hasfocus:
            wx.PostEvent(self, ValueChangedEvent(self.GetId(), value=self.text.GetValue()))
            if wx.Platform == '__WXMAC__':
                wx.Yield()
        self.hasfocus = False



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy
import vtk
from vtk.util.numpy_support import numpy_to_vtk
from .wxVTKRenderWindowInteractor import wxVTKRenderWindowInteractor
import wx

from .visanplot import vtkInteractorStylePlot, vtkPlotActor, vtkXYPlotData

PlotDataChangedEvent, EVT_PLOTDATA_CHANGED = wx.lib.newevent.NewEvent()
PlotAxisChangedEvent, EVT_PLOTAXIS_CHANGED = wx.lib.newevent.NewEvent()

X_AXIS = 0
Y_AXIS = 1

COLORMAP = [[1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0], [1.0, 0.0, 1.0],
            [0.0, 1.0, 1.0], [0.7, 0.7, 0.0], [0.0, 0.7, 0.7], [0.7, 0.0, 0.7],
            [0.5, 0.0, 0.0], [0.0, 0.5, 0.0], [0.0, 0.0, 0.5]]


class PlotWindow(wxVTKRenderWindowInteractor):

    def __init__(self, *args, **kwargs):
        super(PlotWindow, self).__init__(*args, **kwargs)
        # TODO: Still needed?
        # self.UseCaptureMouseOn()
        self.keyframe = 0
        self.numKeyframes = 1

        self.dataSets = []
        self.dataSetProperties = []

        self.actor = vtkPlotActor()
        self.actor.GetProperty().SetColor(0.0, 0.0, 0.0)

        def onPlotAxisChanged(caller, event, calldata=None):
            wx.PostEvent(self, PlotAxisChangedEvent())
        self.actor.AddObserver("XRangeChanged", onPlotAxisChanged)
        self.actor.AddObserver("YRangeChanged", onPlotAxisChanged)

        renderer = vtk.vtkRenderer()
        renderer.SetBackground(1.0, 1.0, 1.0)
        renderer.AddActor2D(self.actor)

        self.GetRenderWindow().AddRenderer(renderer)
        self.GetRenderWindow().SetPointSmoothing(True)
        self.GetRenderWindow().SetLineSmoothing(True)

        istyle = vtkInteractorStylePlot()
        self.SetInteractorStyle(istyle)

    def SetPlotTitle(self, title):
        self.actor.SetTitle(title)
        self.Refresh()

    def GetPlotTitle(self):
        return self.actor.GetTitle()

    def AddDataSet(self, xdata, ydata, dataSetId=None):
        xdata = numpy_to_vtk(numpy.asarray(xdata, dtype=numpy.double))
        ydata = numpy_to_vtk(numpy.asarray(ydata, dtype=numpy.double))
        if dataSetId is None:
            plotData = vtkXYPlotData()
            plotData.AddData(xdata, ydata)
            plotProperty = vtk.vtkProperty2D()
            plotProperty.SetColor(COLORMAP[len(self.dataSets) % 11]);
            self.actor.AddData(plotData, plotProperty)
            self.dataSets.append(plotData)
            self.dataSetProperties.append(plotProperty)
            dataSetId = len(self.dataSets) - 1
        else:
            self.dataSets[dataSetId].AddData(xdata, ydata)
            if self.dataSets[dataSetId].GetNumberOfKeyframes() > self.numKeyframes:
                self.numKeyframes = self.dataSets[dataSetId].GetNumberOfKeyframes()
            self.actor.CalculateDataRanges()
        self.Refresh()

        wx.PostEvent(self, PlotDataChangedEvent())

        return dataSetId

    def UpdateDataSet(self, dataSetId, xdata, ydata):
        xdata = numpy_to_vtk(numpy.asarray(xdata, dtype=numpy.double))
        ydata = numpy_to_vtk(numpy.asarray(ydata, dtype=numpy.double))
        self.dataSets[dataSetId].SetData(xdata, ydata)
        # recalculate number of keyframes
        self.numKeyframes = max(dataSet.GetNumberOfKeyframes() for dataSet in self.dataSets)
        self.actor.CalculateDataRanges()
        self.Refresh()

        wx.PostEvent(self, PlotDataChangedEvent())

        return dataSetId

    def GetDataXRange(self):
        return self.actor.GetDataXRange()

    def GetDataYRange(self):
        return self.actor.GetDataYRange()

    def GetNumDataSets(self):
        return len(self.dataSets)

    def GetNumKeyframes(self):
        return self.numKeyframes

    def GetNumKeyframesForDataSet(self, dataSetId):
        return self.dataSets[dataSetId].GetNumberOfKeyframes()

    def SetKeyframe(self, keyframe):
        if keyframe >= self.numKeyframes:
            keyframe = sef.numKeyframes - 1
        if keyframe < 0:
            keyframe = 0
        self.keyframe = keyframe
        for dataSet in self.dataSets:
            dataSet.SetKeyframe(keyframe)
        self.Refresh()

    def GetKeyframe(self):
        return self.keyframe

    def SetAxisRange(self, axisId, minValue, maxValue):
        if axisId == X_AXIS:
            self.actor.SetXRange(minValue, maxValue)
        if axisId == Y_AXIS:
            self.actor.SetYRange(minValue, maxValue)
        self.Refresh()

    def GetAxisRangeMin(self, axisId):
        if axisId == X_AXIS:
            return self.actor.GetXRange()[0]
        if axisId == Y_AXIS:
            return self.actor.GetYRange()[0]

    def GetAxisRangeMax(self, axisId):
        if axisId == X_AXIS:
            return self.actor.GetXRange()[1]
        if axisId == Y_AXIS:
            return self.actor.GetYRange()[1]

    def SetLogAxis(self, axisId, log):
        if axisId == X_AXIS:
            self.actor.SetLogX(log)
        if axisId == Y_AXIS:
            self.actor.SetLogY(log)
        self.Refresh()

    def GetLogAxis(self, axisId):
        if axisId == X_AXIS:
            return self.actor.GetLogX() != 0
        if axisId == Y_AXIS:
            return self.actor.GetLogY() != 0

    def SetAxisBase(self, axisId, base):
        if axisId == X_AXIS:
            self.actor.SetBaseX(base)
        if axisIdd == Y_AXIS:
            self.actor.SetBaseY(base)
        self.Refresh()

    def GetAxisBase(self, axisId):
        if axisId == X_AXIS:
            return self.actor.GetBaseX()
        if axisId == Y_AXIS:
            return self.actor.GetBaseY()

    def SetNumAxisLabels(self, axisId, numLabels):
        if axisId == X_AXIS:
            self.actor.SetNumberOfXLabels(numLabels)
        if axisId == Y_AXIS:
            self.actor.SetNumberOfYLabels(numLabels)
        self.Refresh()

    def GetNumAxisLabels(self, axisId):
        if axisId == X_AXIS:
            return self.actor.GetNumberOfXLabels()
        if axisId == Y_AXIS:
            return self.actor.GetNumberOfYLabels()

    def SetAxisTitle(self, axisId, title):
        if axisId == X_AXIS:
            self.actor.SetXTitle(title)
        if axisId == Y_AXIS:
            self.actor.SetYTitle(title)
        self.Refresh()

    def GetAxisTitle(self, axisId):
        if axisId == X_AXIS:
            return self.actor.GetXTitle()
        if axisId == Y_AXIS:
            return self.actor.GetYTitle()
        return ""

    def SetAxisLabelFormat(self, axisId, format):
        if axisId == X_AXIS:
            self.actor.SetLabelXFormat(format)
        if axisId == Y_AXIS:
            self.actor.SetLabelYFormat(format)
        self.Refresh()

    def GetAxisLabelFormat(self, axisId):
        if axisId == X_AXIS:
            return self.actor.GetLabelXFormat()
        if axisId == Y_AXIS:
            return self.actor.GetLabelYFormat()

    def SetDataSetLabel(self, dataSetId, label):
        self.dataSets[dataSetId].SetPlotLabel(label)
        self.Refresh()

    def GetDataSetLabel(self, dataSetId):
        return self.dataSets[dataSetId].GetPlotLabel()

    def SetPlotLines(self, dataSetId, plotLines):
        self.dataSets[dataSetId].SetPlotLines(plotLines)
        self.Refresh()

    def GetPlotLines(self, dataSetId):
        return self.dataSets[dataSetId].GetPlotLines()

    def SetLineWidth(self, dataSetId, lineWidth):
        self.dataSetProperties[dataSetId].SetLineWidth(lineWidth)
        self.Refresh()

    def GetLineWidth(self, dataSetId):
        return self.dataSetProperties[dataSetId].GetLineWidth()

    def SetLineStipplePattern(self, dataSetId, stipplePattern):
        self.dataSetProperties[dataSetId].SetLineStipplePattern(stipplePattern)
        self.Refresh()

    def GetLineStipplePattern(self, dataSetId):
        return self.dataSetProperties[dataSetId].GetLineStipplePattern()

    def SetPlotPoints(self, dataSetId, plotPoints):
        self.dataSets[dataSetId].SetPlotPoints(plotPoints)
        self.Refresh()

    def GetPlotPoints(self, dataSetId):
        return self.dataSets[dataSetId].GetPlotPoints()

    def SetPointSize(self, dataSetId, pointSize):
        self.dataSets[dataSetId].SetGlyphSize(pointSize / 100.0)

    def GetPointSize(self, dataSetId):
        return self.dataSets[dataSetId].GetGlyphSize() * 100.0

    def SetPlotColor(self, dataSetId, rgb):
        self.dataSetProperties[dataSetId].SetColor(rgb[0], rgb[1], rgb[2])
        self.Refresh()

    def GetPlotColor(self, dataSetId):
        return self.dataSetProperties[dataSetId].GetColor()

    def SetOpacity(self, dataSetId, opacity):
        self.dataSetProperties[dataSetId].SetOpacity(opacity)
        self.Refresh()

    def GetOpacity(self, dataSetId):
        return self.dataSetProperties[dataSetId].GetOpacity()

    def ExportToImageFile(self, filename, format):
        if format == "tif":
            writer = vtk.vtkTIFFWriter()
        elif format == "bmp":
            writer = vtk.vtkBMPWriter()
        elif format == "jpg":
            writer = vtk.vtkJPEGWriter()
        elif format == "png":
            writer = vtk.vtkPNGWriter()
        elif format == "pnm":
            writer = vtk.vtkPNMWriter()
        else:
            raise Exception("Unsupported export format")

        imageFilter = vtk.vtkWindowToImageFilter()
        imageFilter.SetInput(self.GetRenderWindow())
        writer.SetInputConnection(imageFilter.GetOutputPort())
        writer.SetFileName(filename)
        self.GetRenderWindow().Render()
        writer.Write()



# This is an example VISAN script for the ALD_U_N_2B product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing ALD_U_N_2B products.

# This example will then take the first product it finds in this directory and
# for this product plot the Mie and Rayleigh wind profiles for measurements and observations


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all DBL files in productdir containing with 'ALD_U_N_2B'
    files = glob.glob(os.path.join(productdir, "*ALD_U_N_2B*.DBL"))
    if len(files) == 0:
        print(("Could not find any ALD_U_N_2B files in directory '" + productdir + "'"))
        return

    # We only take the first file
    filename = files[0]

    # Rayleigh (default)
    data = harp.import_product(filename)
    plot(data, title="ALD_U_N_2B Rayleigh Observations", xrange=[-5e4, 5e4])

    # Mie
    data = harp.import_product(filename, "", "data=mie")
    plot(data, title="ALD_U_N_2B Mie Observations", xrange=[-5e4, 5e4])

    # Rayleigh, but only the valid measurements (as a plain list of points)
    data = harp.import_product(filename, "flatten(vertical);hlos_wind_velocity_validity==1")
    plot(data.latitude.data, data.hlos_wind_velocity.data, points=1, lines=0, pointsize=0.1, xlabel="latitude", ylabel=data.hlos_wind_velocity.unit, title="ALD_U_N_2B Rayleigh Observations (points)")

    # Mie, but only the valid measurements (as a plain list of points)
    data = harp.import_product(filename, "flatten(vertical);hlos_wind_velocity_validity==1", "data=mie")
    plot(data.latitude.data, data.hlos_wind_velocity.data, points=1, lines=0, pointsize=0.1, xlabel="latitude", ylabel=data.hlos_wind_velocity.unit, title="ALD_U_N_2B Mie Observations (points)")


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""
Mapping of HARP Product data to Plot input
"""

import re
import coda
import harp
import numpy


UNPREFERED_PATTERNS = [
    "index", "collocation_index", "orbit_index", ".*subindex", "scan_direction_type",
    "datetime.*",
    "sensor_.*",
    ".*validity",
    ".*_uncertainty.*",
    ".*_apriori.*",
    ".*_amf.*",
    "wavenumber$",
    "wavelength$",
    ".*latitude.*",
    ".*longitude.*",
    ".*altitude",
    ".*geoid_height",
    ".*geopotential",
    ".*pressure",
    ".*_angle"
]


def get_prefered_value(values, unprefered_patterns):
    if unprefered_patterns:
        value = get_prefered_value([value for value in values if not re.match(unprefered_patterns[0], value)],
                                   unprefered_patterns[1:])
        if value is not None:
            return value
    if values:
        return values[0]
    return None


def get_attributes(product):
    # we return all scalars and 1D (time dependent) variables
    def attr_value(value, variable):
        if hasattr(variable, "unit"):
            if " since " in variable.unit:
                # this is a time value
                base, epoch = variable.unit.split(" since ")
                if base in ["s", "seconds", "days"]:
                    if base == "days":
                        value *= 86400
                    formats = "yyyy-MM-dd HH:mm:ss.SSSSSS|yyyy-MM-dd HH:mm:ss|yyyy-MM-dd"
                    value = value + coda.time_string_to_double(formats, epoch)
                    return coda.time_to_string(value)
            return "%s [%s]" % (str(value), variable.unit)
        return str(value)

    attr = {}
    for name in list(product):
        if len(product[name].dimension) == 0:
            attr[name] = attr_value(product[name].data, product[name])
        elif len(product[name].dimension) == 1 and product[name].dimension[0] == 'time':
            attr[name] = [attr_value(value, product[name]) for value in product[name].data]
    return attr


def get_midpoint_axis_from_bounds(bounds_variable, log=False):
    if bounds_variable.data.shape[-1] != 2 or bounds_variable.dimension[-1] is not None:
        raise ValueError("bounds variable should end with independent dimension of length 2")
    if log:
        data = numpy.exp((numpy.log(bounds_variable.data[..., 0]) + numpy.log(bounds_variable.data[..., 1])) / 2.0)
    else:
        data = (bounds_variable.data[..., 0] + bounds_variable.data[..., 1]) / 2.0
    return harp.Variable(data, bounds_variable.dimension[:-1], bounds_variable.unit)


def PlotDataForProduct(product, value=None):
    if not isinstance(product, harp.Product):
        raise TypeError("Expecting a HARP product")

    variable_names = []

    for name in list(product):
        if len(product[name].dimension) == 0 or len(product[name].dimension) > 2:
            continue
        if not isinstance(product[name].data, (numpy.ndarray, numpy.generic)):
            continue
        if product[name].data.dtype.char in ['O', 'S']:
            continue
        if product[name].dimension[0] != 'time':
            continue
        if len(product[name].dimension) == 2 and product[name].dimension[1] not in ['spectral', 'vertical']:
            continue
        variable_names += [name]

    if value is not None:
        if value not in list(product):
            raise ValueError("product variable does not exist ('%s')" % value)
        if value not in variable_names:
            raise ValueError("product variable is not plottable ('%s')" % value)
    else:
        value = get_prefered_value(variable_names, UNPREFERED_PATTERNS)

    if value is None:
        raise ValueError("HARP product is not plotable")

    xdata = None
    ydata = product[value]
    attr = {}
    location = []
    prop = {'title': value.replace('_', ' '), 'name': value}

    if len(product[value].dimension) == 2:
        if product[value].dimension[1] == 'spectral':
            if 'wavelength' in product:
                xdata = product['wavelength']
            elif 'wavenumber' in product:
                xdata = product['wavenumber']
            if xdata is None:
                raise ValueError("Could not determine x-axis for spectral data ('%s')" % value)
        else:
            # product[value].dimension[1] == 'vertical'
            # swap axis
            xdata = ydata
            ydata = None
            if 'altitude' in product:
                ydata = product['altitude']
            elif 'altitude_bounds' in product:
                ydata = get_midpoint_axis_from_bounds(product['altitude_bounds'])
            elif 'pressure' in product:
                ydata = product['pressure']
                prop["ylog"] = True
            elif 'pressure_bounds' in product:
                ydata = get_midpoint_axis_from_bounds(product['pressure_bounds'], log=True)
                prop["ylog"] = True
            if ydata is None:
                raise ValueError("Could not determine y-axis for vertical profile data ('%s')" % value)
        attr = get_attributes(product)
    else:
        if 'datetime' in product:
            xdata = product['datetime']
        elif 'datetime_start' in product:
            xdata = product['datetime_start']
        elif 'datetime_stop' in product:
            xdata = product['datetime_stop']
        if xdata is None:
            raise ValueError("Could not determine x-axis for time-series data ('%s')" % value)

    if 'latitude_bounds' in product and 'longitude_bounds' in product:
        location = [product.latitude_bounds.data, product.longitude_bounds.data]
    elif 'latitude' in product and 'longitude' in product:
        location = [product.latitude.data, product.longitude.data]

    try:
        prop['xlabel'] = xdata.unit
    except AttributeError:
        pass
    try:
        prop['ylabel'] = ydata.unit
    except AttributeError:
        pass

    return (xdata.data, ydata.data, attr, location, prop)


kPointData = 0
kSwathData = 1
kGridData = 2


def WorldPlotDataForProduct(product, locationOnly, value=None):
    if not isinstance(product, harp.Product):
        raise TypeError("Expecting a HARP product")

    data_type = kPointData
    latitude = None
    longitude = None
    if 'latitude_bounds' in list(product) and 'longitude_bounds' in list(product):
        latitude_bounds = product['latitude_bounds']
        longitude_bounds = product['longitude_bounds']
        if len(latitude_bounds.dimension) != len(longitude_bounds.dimension):
            raise ValueError("latitude and longitude bounds should have same number of dimensions")
        if latitude_bounds.dimension[-1] is not None or longitude_bounds.dimension[-1] is not None:
            raise ValueError("last dimension for latitude and longitude bounds should be independent")
        if len(latitude_bounds.dimension) > 1 and latitude_bounds.dimension[-2] == 'latitude' and \
                longitude_bounds.dimension[-2] == 'longitude':
            if len(latitude_bounds.dimension) == 3:
                if latitude_bounds.dimension[0] != 'time' or longitude_bounds.dimension[0] != 'time':
                    raise ValueError("first dimension for latitude and longitude bounds should be the time dimension")
            else:
                if len(latitude_bounds.dimension) != 2:
                    raise ValueError("latitude and longitude bounds should be "
                                     "two or three dimensional for gridded data")
            if latitude_bounds.data.shape[-1] != 2 or longitude_bounds.data.shape[-1] != 2:
                raise ValueError("independent dimension of latitude and longitude bounds should have "
                                 "length 2 for gridded data")
            data_type = kGridData
            latitude = get_midpoint_axis_from_bounds(latitude_bounds)
            longitude = get_midpoint_axis_from_bounds(longitude_bounds)
        elif not locationOnly:
            if len(latitude_bounds.dimension) != 2:
                raise ValueError("latitude and longitude bounds should be two dimensional for non-gridded data")
            if latitude_bounds.dimension[0] != 'time' or longitude_bounds.dimension[0] != 'time':
                raise ValueError("first dimension for latitude and longitude bounds should be the time dimension")
            if latitude_bounds.data.shape != longitude_bounds.data.shape:
                raise ValueError("latitude and longitude bounds should have the same dimension lengths")
            data_type = kSwathData
            latitude = latitude_bounds
            longitude = longitude_bounds
    if data_type != kSwathData and 'latitude' in list(product) and 'longitude' in list(product):
        latitude = product['latitude']
        longitude = product['longitude']
        if len(latitude.dimension) != len(longitude.dimension):
            raise ValueError("latitude and longitude should have same number of dimensions")
        if len(latitude.dimension) > 0 and latitude.dimension[-1] == 'latitude' and \
                longitude.dimension[-1] == 'longitude':
            # if we have both lat/lon center and lat/lon bounds then use the center lat/lon for the grid axis
            if len(latitude.dimension) == 2:
                if latitude.dimension[0] != 'time' or longitude.dimension[0] != 'time':
                    raise ValueError("first dimension for latitude and longitude should be the time dimension")
            else:
                if len(latitude.dimension) != 1:
                    raise ValueError("latitude and longitude should be one or two dimensional")
            data_type = kGridData
        else:
            if len(latitude.dimension) != 1:
                raise ValueError("latitude and longitude should be one dimensional")
            if latitude.dimension[0] != 'time' or longitude.dimension[0] != 'time':
                raise ValueError("first dimension for latitude and longitude should be the time dimension")
            data_type = kPointData
    if latitude is None or longitude is None:
        raise ValueError("HARP product has no latitude/longitude information")

    if locationOnly:
        value = None
    else:
        variable_names = []

        for name in list(product):
            if not isinstance(product[name].data, (numpy.ndarray, numpy.generic)):
                continue
            if product[name].data.dtype.char in ['O', 'S']:
                continue
            if data_type == kGridData:
                if len(product[name].dimension) < 2 or len(product[name].dimension) > 3:
                    continue
                if product[name].dimension[-2] != 'latitude':
                    continue
                if product[name].dimension[-1] != 'longitude':
                    continue
                if len(product[name].dimension) == 3 and product[name].dimension[0] != 'time':
                    continue
            else:
                if len(product[name].dimension) != 1:
                    continue
                if product[name].dimension[0] != 'time':
                    continue
            variable_names += [name]

        if value is not None:
            if value not in list(product):
                raise ValueError("product variable does not exist ('%s')" % value)
            if value not in variable_names:
                raise ValueError("product variable is not plottable ('%s')" % value)
        else:
            value = get_prefered_value(variable_names, UNPREFERED_PATTERNS)

    data = None
    attr = {}
    prop = {}

    if data_type == kGridData:
        attr = get_attributes(product)
        if value is None:
            raise ValueError("HARP product has no variable to use for gridded plot")

    if value is not None:
        data = product[value].data
        prop["name"] = value
        prop["colorbartitle"] = value.replace('_', ' ')
        try:
            prop["colorbartitle"] += " [" + product[value].unit + "]"
        except AttributeError:
            pass

    return (data_type, data, latitude.data, longitude.data, attr, prop)



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os

import wx


class IntroFrame(wx.Frame):

    def __init__(self, parent):
        wx.Frame.__init__(self, parent, -1, "Getting Started with VISAN")

        if wx.Config.Get().Read("IconFile"):
            self.SetIcon(wx.Icon(wx.Config.Get().Read("IconFile")))

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        intropanel = IntroPanel(self)
        self.sizer.Add(intropanel, 1, wx.EXPAND)

        self.SetSizerAndFit(self.sizer)
        self.CentreOnParent()


class IntroPanel(wx.Panel):

    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)

        self.CreateControls()
        self.CreateLayout()
        self.InstallToolTips()
        self.InstallEventListeners()

    def CreateControls(self):
        intro = ("Welcome to VISAN!\n\nIf you are new to VISAN you can do any of the following to get you started "
                 "using this application:\n\n"
                 "(a) Have a look at the VISAN documentation. You can open the documentation in your webbrowser by "
                 "chosing 'VISAN Documentation' from the 'Help' menu. The VISAN documentation contains a small "
                 "Tutorial to get you on your way quickly;\n\n"
                 "(b) To learn more about the language (Python) that you use to enter commands in VISAN, have a "
                 "look at the Python documentation which is also accessible from the 'Help' menu. This documentation "
                 "comes with a very extensive Tutorial explaining the basics of the Python language;\n\n"
                 "(c) In the examples directory of your VISAN installation ('" + wx.GetApp().exampledir + "') you "
                 "can find several example scripts. You can examine the contents of a script by opening it in a text "
                 "editor and/or you can run the script in VISAN using the 'Load and Execute Script...' option from "
                 "the 'File' menu.")
        if wx.Platform == '__WXMAC__':
            pointSize = 12
            width = 400
            height = 370
        elif wx.Platform == '__WXMSW__':
            pointSize = 10
            width = 400
            height = 370
        else:
            pointSize = 10
            width = 470
            height = 430
        self.introText = wx.TextCtrl(self, -1, size=(width, height),
                                     style=wx.TE_RICH | wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_WORDWRAP)

        font = wx.Font(pointSize=pointSize, family=wx.FONTFAMILY_DEFAULT, style=wx.FONTSTYLE_NORMAL,
                       weight=wx.FONTWEIGHT_BOLD)
        style = wx.TextAttr(colText=wx.Colour(153, 51, 51), font=font)
        self.introText.SetDefaultStyle(style)
        self.introText.SetEditable(False)
        self.introText.AppendText(intro)
        self.introText.Layout()
        self.againCheckBox = wx.CheckBox(self, -1, "Show this window at next startup")
        self.againCheckBox.SetValue(wx.Config.Get().ReadBool("ShowIntroFrame", True))
        self.againCheckBox.SetFocus()

    def CreateLayout(self):
        self.vsizer = wx.BoxSizer(wx.VERTICAL)
        self.vsizer.Add(self.introText, 1, wx.EXPAND | wx.BOTTOM, border=10)
        self.vsizer.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND)
        self.vsizer.Add(self.againCheckBox, 0, wx.TOP, border=10)

        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer.Add(self.vsizer, 1, wx.ADJUST_MINSIZE | wx.EXPAND | wx.ALL, border=10)

        self.SetAutoLayout(True)
        self.SetSizerAndFit(self.sizer)

        self.introText.ShowPosition(1)

    def InstallToolTips(self):
        self.againCheckBox.SetToolTip(wx.ToolTip("Even if you turn this option off, this window can always be "
                                                 "explicitly displayed via the 'Help' menu"))

    def InstallEventListeners(self):
        self.againCheckBox.Bind(wx.EVT_CHECKBOX, self.OnAgain)

    def OnAgain(self, event):
        wx.Config.Get().WriteBool("ShowIntroFrame", self.againCheckBox.IsChecked())



# This is an example VISAN script for the ALD_U_N_1B product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing ALD_U_N_1B products.

# This example will then take the first product it finds in this directory and
# for this product plot the Mie and Rayleigh wind profiles for measurements and observations


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all DBL files in productdir containing with 'ALD_U_N_1B'
    files = glob.glob(os.path.join(productdir, "*ALD_U_N_1B*.DBL"))
    if len(files) == 0:
        print(("Could not find any ALD_U_N_1B files in directory '" + productdir + "'"))
        return

    # We only take the first file
    filename = files[0]

    # Since the L1b data only has altitude boundaries we need to perform a
    # derive(altitude {time,vertical} [km]) to get the center altitude values for plotting
    operations = "derive(altitude {time,vertical} [km])"
    # We also turn the lat/lon values per profile point to single lat/lon values per profile
    # This allows us to show the location of each profile in the plot property panel
    operations += ";derive(latitude {time});derive(longitude {time})"

    # Rayleigh Observations (default)
    data = harp.import_product(filename, operations)
    plot(data, title="ALD_U_N_1B Rayleight Observations", xrange=[-500, 500])

    # Mie Observations
    data = harp.import_product(filename, operations, "data=mie_observation")
    plot(data, title="ALD_U_N_1B Mie Observations", xrange=[-500, 500])

    # Rayleigh Measuruments
    data = harp.import_product(filename, operations, "data=rayleigh_measurement")
    plot(data, title="ALD_U_N_1B Rayleight Measuruments", xrange=[-500, 500])

    # Mie Measurements
    data = harp.import_product(filename, operations, "data=mie_measurement")
    plot(data, title="ALD_U_N_1B Mie Measuruments", xrange=[-500, 500])


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import wx

defaultsize = (800, 494)
locx = 50
locy = 50
offset = 30


def GetDefaultSize():
    return defaultsize


def SetPosition(pos):
    global locx, locy
    locx = pos[0]
    locy = pos[1]


def GetNextPosition(size=defaultsize):
    global locx, locy

    xmax = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_X)
    ymax = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_Y)
    locx += offset
    locy += offset

    # Test if win stays within window
    if (size[0] + locx) > xmax - offset or \
       (size[1] + locy) > ymax - offset:
        locx = offset
        locy = offset

    return (locx, locy)



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import wx

from .labeledtextctrl import LabeledTextCtrl, EVT_VALUE_CHANGED
from .util import DetermineCharSize
from .validators import DigitValidator, FloatValidator


class ColorBarPanel(wx.Panel):

    def __init__(self, parent, plotWindow):
        panelstyle = wx.TAB_TRAVERSAL
        if wx.Platform == '__WXGTK__':
            panelstyle |= wx.SUNKEN_BORDER
        wx.Panel.__init__(self, parent, -1, style=panelstyle)

        # Set initial state
        self.plotWindow = plotWindow

        # Create and configure all widgets
        self.CreateControls()
        self.CreateLayout()

    def CreateControls(self):
        (charWidth, charHeight) = DetermineCharSize(self)

        self.titleCtrl = LabeledTextCtrl(self, -1, label="Color Bar Title:", value="",
                                         size=(10 * charWidth, -1), style=wx.TE_PROCESS_ENTER)
        self.titleCtrl.SetToolTip(wx.ToolTip("This title will be shown just above the color bar and should "
                                             "contain information on the physical unit."))
        self.titleCtrl.Bind(EVT_VALUE_CHANGED, self.OnTitle)

        self.numLabelsCtrl = LabeledTextCtrl(self, -1, label="Number of labels:", validator=DigitValidator(),
                                             size=(3 * charWidth, -1), style=wx.TE_PROCESS_ENTER)
        self.numLabelsCtrl.SetToolTip(wx.ToolTip("Number of labels shown on the plot's color bar."))
        self.numLabelsCtrl.Bind(EVT_VALUE_CHANGED, self.OnNumLabels)

        # static boxes should be created before the controls they contain in order to preserve the correct Z-Order
        self.rangeBox = wx.StaticBox(self, -1, "Range")

        self.rangeMinCtrl = LabeledTextCtrl(self, -1, label="Min:", value="0", validator=FloatValidator(),
                                            formatstring="%g", size=(5 * charWidth, -1),
                                            style=wx.TE_PROCESS_ENTER)
        self.rangeMinCtrl.SetToolTip(wx.ToolTip("Minimium value to use for the color range."))
        self.rangeMinCtrl.Bind(EVT_VALUE_CHANGED, self.OnRange)

        self.rangeMaxCtrl = LabeledTextCtrl(self, -1, label="Max:", value="1", formatstring="%g",
                                            validator=FloatValidator(), size=(5 * charWidth, -1),
                                            style=wx.TE_PROCESS_ENTER)
        self.rangeMaxCtrl.SetToolTip(wx.ToolTip("Maximum value to use for the color range."))
        self.rangeMaxCtrl.Bind(EVT_VALUE_CHANGED, self.OnRange)

        self.UpdateControls()

    def CreateLayout(self):
        spacing = 10
        if wx.Platform == '__WXMAC__':
            spacing = 6

        self.hsizer3 = wx.BoxSizer(wx.HORIZONTAL)
        self.hsizer3.Add((spacing, 0), 0)
        self.hsizer3.Add(self.rangeMinCtrl, 0, wx.RIGHT, border=spacing)
        self.hsizer3.Add(self.rangeMaxCtrl, 0)

        self.vsizer2 = wx.StaticBoxSizer(self.rangeBox, wx.VERTICAL)
        self.vsizer2.Add((0, spacing), 0)
        self.vsizer2.Add(self.hsizer3, 1, wx.LEFT | wx.RIGHT, border=spacing // 2)
        self.vsizer2.Add((0, spacing), 0)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.titleCtrl, 0, wx.EXPAND | wx.ALL, border=spacing // 2)
        self.sizer.Add((0, spacing), 0)
        self.sizer.Add(self.numLabelsCtrl, 0, wx.LEFT | wx.RIGHT, border=spacing // 2)
        self.sizer.Add((0, spacing), 0)
        self.sizer.Add(self.vsizer2, 0, wx.EXPAND)

        self.mainsizer = wx.BoxSizer(wx.VERTICAL)
        self.mainsizer.Add(self.sizer, 1, wx.EXPAND | wx.ALL, border=spacing)

        self.SetSizerAndFit(self.mainsizer)

    def UpdateControls(self, dataSetId=None):
        if dataSetId is None:
            dataSetId = self.plotWindow.GetDataSetForColorBar()
        if dataSetId >= 0:
            self.titleCtrl.SetValue(self.plotWindow.GetColorBarTitle(dataSetId))
            self.colorRange = self.plotWindow.GetColorRange(dataSetId)
            self.numLabels = self.plotWindow.GetNumColorBarLabels(dataSetId)
        else:
            self.titleCtrl.SetValue("")
            self.colorRange = [0, 1]
            self.numLabels = 2
        self.numLabelsCtrl.SetValue(self.numLabels)
        self.rangeMinCtrl.SetValue(self.colorRange[0])
        self.rangeMaxCtrl.SetValue(self.colorRange[1])

        # Update enabled state of text fields
        self.titleCtrl.Enable(dataSetId >= 0)
        self.numLabelsCtrl.Enable(dataSetId >= 0)
        self.rangeMinCtrl.Enable(dataSetId >= 0)
        self.rangeMaxCtrl.Enable(dataSetId >= 0)

    def OnTitle(self, event):
        dataSetId = self.plotWindow.GetDataSetForColorBar()
        if dataSetId >= 0:
            curTitle = str(self.plotWindow.GetColorBarTitle(dataSetId))
            newTitle = str(self.titleCtrl.GetValue())
            # Check if the new value is different
            if newTitle != curTitle:
                self.plotWindow.SetColorBarTitle(dataSetId, str(self.titleCtrl.GetValue()))

    def OnNumLabels(self, event):
        try:
            value = int(self.numLabelsCtrl.GetValue())
        except ValueError:
            wx.Bell()
            self.numLabelsCtrl.SetValue(self.numLabels)
            return
        if value < 0:
            wx.Bell()
            self.numLabelsCtrl.SetValue(self.numLabels)
            return
        self.numLabels = value
        dataSetId = self.plotWindow.GetDataSetForColorBar()
        if dataSetId >= 0:
            self.plotWindow.SetNumColorBarLabels(dataSetId, self.numLabels)

    def OnRange(self, event):
        try:
            value = [float(self.rangeMinCtrl.GetValue()), float(self.rangeMaxCtrl.GetValue())]
        except ValueError:
            wx.Bell()
            self.rangeMinCtrl.SetValue(self.colorRange[0])
            self.rangeMaxCtrl.SetValue(self.colorRange[1])
            return
        if value[0] > value[1]:
            wx.Bell()
            self.rangeMinCtrl.SetValue(self.colorRange[0])
            self.rangeMaxCtrl.SetValue(self.colorRange[1])
            return
        self.colorRange = value
        dataSetId = self.plotWindow.GetDataSetForColorBar()
        if dataSetId >= 0:
            self.plotWindow.SetColorRange(dataSetId, self.colorRange[0], self.colorRange[1])



# This is an example VISAN script to create a daily grid of S5P NO2 L2 product data

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing one day of S5P NO2 L2 products.

# This example will then take all products it finds in this directory
# create a global grid from it and plot the resulting daily grid of tropospheric NO2 columns


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all S5P NO2 files in productdir
    files = glob.glob(os.path.join(productdir, "S5P_????_L2__NO2*"))

    if len(files) == 0:
        print(("Could not find any S5P NO2 L2 files in directory '" + productdir + "'"))
        return

    # Filter for quality, turn each product into a lat/lon grid and merge these grids into a single daily grid
    data = harp.import_product(files, 'tropospheric_NO2_column_number_density_validity>75;keep(datetime_start,datetime_length,latitude_bounds,longitude_bounds,tropospheric_NO2_column_number_density);bin_spatial(1801,-90,0.1,3601,-180,0.1);derive(latitude {latitude});derive(longitude {longitude});exclude(latitude_bounds,longitude_bounds)', post_operations='bin();squash(time, (latitude,longitude));derive(tropospheric_NO2_column_number_density [Pmolec/cm2])')

    wplot(data, value="tropospheric_NO2_column_number_density", colorrange=(0,20), showpropertypanel=True)


run()



# This is an example VISAN script for the SCI_OL__2P product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing SCI_OL__2P products.

# This example will then take all products it finds in this directory and
# for these products plot the ozone profiles from the limb mode


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all files in productdir starting with 'SCI_OL__2P'
    files = glob.glob(os.path.join(productdir, "SCI_OL__2P*"))
    if len(files) == 0:
        print(("Could not find any SCI_OL__2P files in directory '" + productdir + "'"))
        return

    # We ingest the ozone profile data that was retrieved using UV/VIS fitting window no. 0.
    o3 = harp.import_product(files, "", "dataset=lim_uv0_o3")

    wplot(o3, projection="Plate Caree", pointsize=3, colortable='RedToGreen')
    window = plot(o3, title="SCI_OL__2P profile example (o3)")


run()



# This is an example VISAN script that displays Sun Reference Spectra from
# a SCIAMACHY Level 1b product.

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing SCI_NL__1P products.

# This example will then take the first product it finds in this directory and
# for that product plot the sun reference spectra


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all SCIAMACHY L1b files in productdir.
    files = glob.glob(os.path.join(productdir, "SCI_NL__1P*"))
    if len(files) == 0:
        print(("Could not find any SCI_NL__1P files in directory '" + productdir + "'"))
        return

    # Ingest the ozone profile and tangent altitudes from the first SCIAMACHY
    # product file.

    pf = coda.open(files[0])
    if coda.get_product_type(pf) != 'SCI_NL__1P':
        print((files[0] + "is not a SCI_NL__1P file"))
        coda.close(pf)
        return

    num_sun_reference = coda.get_size(pf, 'sun_reference')

    for i in range(num_sun_reference[0]):
        wvlen = coda.fetch(pf, 'sun_reference', i, 'wvlen_sun_meas')

        # value '999' signifies invalid data.
        print((wvlen[0, 0]))
        if wvlen[0, 0] != 999:
            spectrum = coda.fetch(pf, 'sun_reference', i, 'mean_ref_spec')
            w = plot(wvlen[0], spectrum[0], color=(0, 0, 1))
            plot(wvlen[1], spectrum[1], color=(0, 1, 0), window=w)
            plot(wvlen[2], spectrum[2], color=(1, 0, 0), window=w)
            plot(wvlen[3], spectrum[3], color=(1, 0, 1), window=w)
            plot(wvlen[4], spectrum[4], color=(0, 0, 1), window=w)
            plot(wvlen[5], spectrum[5], color=(0, 1, 0), window=w)
            plot(wvlen[6], spectrum[6], color=(1, 0, 0), window=w)
            plot(wvlen[7], spectrum[7], color=(1, 0, 1), window=w, xrange=(200, 2400), ylog=1,
                 xlabel="wavelength [ nm ]", ylabel="photons/\n(m^2.nm.s)",
                 title="GADS sun reference spectrum (GADS %d)" % i)

    coda.close(pf)


run()



# This is an example VISAN script for the lfit() function.

from visan.math import lfit


def run():

    # Create 5 sample points
    x = numpy.array([0, 1, 2, 3, 4])
    y = numpy.array([1, 3, 2, 5, 4])
    w = plot(x, y, lines=False, points=True, name="measurements")

    # We are going to plot our fitted functions using 50 points in the range [0,5)
    xh = numpy.arange(50.) / 10

    # Fit the sample points using the function:
    #     y = a1 + a2 * x
    def F(x):
        return numpy.array([numpy.ones(numpy.shape(x), dtype=float), x])
    a = lfit(x, y, F)
    plot(xh, numpy.dot(a, F(xh)), window=w, name="first order fit")

    # Fit the sample points using the function:
    #     y = a1 + a2 * x + a2 * x^2
    def F(x):
        return numpy.array([numpy.ones(numpy.shape(x), dtype=float), x, x * x])
    a = lfit(x, y, F)
    plot(xh, numpy.dot(a, F(xh)), window=w, name="second order fit")

    # Fit the sample points using the function:
    #     y = a1 + a2 * x + a2 * x^2 + a3 * x^3
    def F(x):
        return numpy.array([numpy.ones(numpy.shape(x), dtype=float), x, x * x, x ** 3])
    a = lfit(x, y, F)
    plot(xh, numpy.dot(a, F(xh)), window=w, name="third order fit")

    # Fit the sample points using the function:
    #     y = a1 + a2 * x + a2 * x^2 + a3 * x^3 + a4 * x^4
    def F(x):
        return numpy.array([numpy.ones(numpy.shape(x), dtype=float), x, x * x, x ** 3, x ** 4])
    a = lfit(x, y, F)
    plot(xh, numpy.dot(a, F(xh)), window=w, name="fourth order fit", title="lfit() example")


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
import sys
import traceback
import webbrowser

import wx
import wx.adv
import wx.py
from wx.lib.mixins.inspection import InspectionMixin

import harp
import coda

from .frame import VisanFrame
from .intro import IntroFrame
from .harpimport import HarpImportDialog
from .productbrowser import ProductBrowser
from . import windowhandler as WindowHandler

from . import VERSION


class VisanApp(wx.App, InspectionMixin):

    def __init__(self):
        visanhomedir = os.path.dirname(__file__)
        # setup directory locations
        self.homedir = visanhomedir
        self.datadir = os.path.join(visanhomedir, 'data')
        self.exampledir = os.path.join(visanhomedir, 'examples')
        if wx.Platform == '__WXMSW__':
            self.docdir = os.path.join(visanhomedir, 'Doc')
        else:
            self.docdir = os.path.join(visanhomedir, 'doc')
        self.htmldir = os.path.join(self.docdir, 'html')

        wx.App.__init__(self, redirect=(sys.platform == 'win32'))

        wx.SystemOptions.SetOption("mac.listctrl.always_use_generic", "1")

        # add current working directory to sys.path (if applicable)
        if wx.Platform == '__WXGTK__':
            sys.path.insert(0, os.getcwd())
        elif wx.Platform == '__WXMSW__':
            if os.getcwd() != sys.prefix + '\\bin':
                sys.path.insert(0, os.getcwd())

    def OnInit(self):

        # Make sure the file location for our preferences exists
        userDataDir = wx.StandardPaths.Get().GetUserDataDir()
        if not os.path.exists(userDataDir):
            os.mkdir(userDataDir)

        self.productdir = wx.StandardPaths.Get().GetDocumentsDir()

        # Initialize preferences
        configStyle = wx.CONFIG_USE_LOCAL_FILE
        if wx.Platform == '__WXGTK__':
            configStyle |= wx.CONFIG_USE_SUBDIR
        config = wx.Config(appName='visan', style=configStyle)
        config.SetRecordDefaults()
        wx.Config.Set(config)

        # Set some global defaults
        config.Read('UserMode', 'EndUser')
        config.Read('DirectoryLocation/Export', wx.StandardPaths.Get().GetDocumentsDir())
        config.Read('DirectoryLocation/Products', self.productdir)
        config.Read('DirectoryLocation/Scripts', self.exampledir)
        # datadir is not something that the user should change, but it is usefull to be able to get
        # to this parameter using the Config system, so that is why we set it here explicitly
        config.Write('DirectoryLocation/ApplicationData', self.datadir)
        # since icons are platform specific we store the appropriate icon to use in the config
        if wx.Platform == '__WXGTK__':
            config.Write('IconFile', os.path.join(self.datadir, "visan32.ico"))
        else:
            config.Write('IconFile', os.path.join(self.datadir, "visan.ico"))

        if config.Read('UserMode') == 'Developer':
            # allows you to pop up a window with an overview of all the widgets in VISAN
            # the shortcut for this is Ctrl-Alt-I (or Cmd-Alt-I on Mac)
            InspectionMixin.Init(self)

        self.filehistory = wx.FileHistory()
        self.recentfiles = False

        coda.set_option_perform_conversions(config.ReadBool('CODA/PerformConversions', True))
        coda.set_option_filter_record_fields(config.ReadBool('CODA/FilterRecordFields', True))

        self.frame = VisanFrame(self, "VISAN " + VERSION, WindowHandler.GetNextPosition((800, 640)), (800, 640))
        self.SetTopWindow(self.frame)
        self.shell = self.frame.shell

        self._PrefsToFileHistory()

        self._SetupLogging()

        wx.py.dispatcher.connect(receiver=self.CheckForExit, signal='Interpreter.push')

        self.frame.Show(True)

        if wx.Config.Get().ReadBool('ShowIntroFrame', True):
            self.ShowIntro()

        self._CreateSplashScreen()

        if len(sys.argv) > 1:
            # don't treat macos -psn arguments as a startup script
            if not (wx.Platform == '__WXMAC__' and sys.argv[1].startswith('-psn_')):
                self._ExecuteScript(sys.argv[1])

        return True

    def _FileHistoryToPrefs(self):
        if self.recentfiles:
            config = wx.Config.Get()
            if config.HasGroup('RecentScripts'):
                config.DeleteGroup('RecentScripts')
            # we write the files in reverse order in order to balance to reversal caused by _AddFileToHistory
            for i in range(self.filehistory.GetCount()):
                filename = self.filehistory.GetHistoryFile(self.filehistory.GetCount() - i - 1)
                config.Write('RecentScripts/file%d' % i, filename)

    def _PrefsToFileHistory(self):
        config = wx.Config.Get()
        i = 0
        while config.Exists('RecentScripts/file%d' % i):
            self._AddFileToHistory(config.Read('RecentScripts/file%d' % i))
            i += 1

    def _SetupLogging(self):
        self.loggingMode = True
        self.sessionlogFrame = LogWindow(self.frame, -1, "VISAN Session Log", size=(450, 200))
        self.sessionlisting = SessionListing(self.sessionlogFrame)
        self.sessionlisting.Start()

    def CloseLogWindow(self):
        self.sessionlogFrame.Show(False)

    def ShowLog(self):
        self.sessionlogFrame.Raise()
        self.sessionlogFrame.Show(True)

    def ClearLog(self):
        self.sessionlisting.Clear()

    def SaveLog(self):
        filename = wx.FileSelector("Save VISAN/Python Script", default_extension="py",
                                   default_path=wx.Config.Get().Read('DirectoryLocation/Scripts'),
                                   wildcard="VISAN/Python Script|*.py", flags=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
                                   parent=self.frame)
        if filename:
            if not filename.endswith(".py"):
                filename = filename + ".py"
            saved = self.sessionlisting.SaveFile(filename)
            if saved:
                self._AddFileToHistory(filename)
                wx.Config.Get().Write('DirectoryLocation/Scripts', os.path.dirname(filename))

    def ExecuteScript(self):
        filename = wx.FileSelector("Open VISAN/Python Script", default_extension="py",
                                   default_path=wx.Config.Get().Read('DirectoryLocation/Scripts'),
                                   wildcard="VISAN/Python Scripts|*.py|All Files|*",
                                   flags=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST, parent=self.frame)
        if filename:
            self._ExecuteScript(filename)
            self._AddFileToHistory(filename)
            wx.Config.Get().Write('DirectoryLocation/Scripts', os.path.dirname(filename))

    def _ExecuteScript(self, filename):
        self.shell.clearCommand()
        self.shell.interp.more = False
        self.shell.DocumentEnd()
        self.shell.write("execute script: " + filename)
        self.shell.push('executescript(%r, globals())\n' % filename)
        self.shell.DocumentEnd()

    def BrowseProduct(self, filename=None):
        if filename is None:
            filename = wx.FileSelector("Browse Product File",
                                       default_path=wx.Config.Get().Read('DirectoryLocation/Products'),
                                       wildcard="All Files|*", flags=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST,
                                       parent=self.frame)
        if filename:
            try:
                browser = ProductBrowser(self.frame, filename)
            except coda.CodacError as ex:
                message = wx.MessageDialog(self.frame,
                                           "Could not open product.\n\nCODA error message: \"%s\"" % str(ex),
                                           "CODA error", style=wx.OK | wx.ICON_ERROR | wx.STAY_ON_TOP)
                message.ShowModal()
                return
            browser.SetPosition(WindowHandler.GetNextPosition(browser.GetSize()))
            if wx.Config.Get().Read("IconFile"):
                browser.SetIcon(wx.Icon(wx.Config.Get().Read("IconFile")))
            browser.Show()
            wx.Config.Get().Write('DirectoryLocation/Products', os.path.dirname(filename))

    def HarpImport(self, filename=None, operations=None, ingestionOptions=None, variableName=None):
        if filename is None:
            importDialog = HarpImportDialog(self.frame)
            if importDialog.ShowModal() == wx.ID_OK:
                filename = importDialog.GetFilename()
                operations = importDialog.GetOperations()
                ingestionOptions = importDialog.GetIngestionOptions()
                variableName = importDialog.GetVariableName()
            importDialog.Destroy()
        if not filename:
            return
        if variableName:
            self.shell.clearCommand()
            self.shell.interp.more = False
            self.shell.DocumentEnd()
            command = "%s = harp.import_product(r\"%s\"" % (variableName, filename)
            if operations:
                command += ", operations='%s'" % (operations,)
            if ingestionOptions:
                command += ", options='%s'" % (ingestionOptions,)
            command += ")\n"
            self.shell.write(command)
            self.shell.push(command)
            self.shell.DocumentEnd()
        else:
            try:
                data = harp.import_product(filename, operations, ingestionOptions)
            except harp.Error as ex:
                message = wx.MessageDialog(self.frame,
                                           "Could not import product(s).\n\nHARP error message: \"%s\"" % str(ex),
                                           "HARP error", style=wx.OK | wx.ICON_ERROR | wx.STAY_ON_TOP)
                message.ShowModal()
                return

    def _AddFileToHistory(self, filename):
        if not self.recentfiles:
            self.recentfiles = True
            self.frame.recentMenuItem.Enable(True)

        self.filehistory.AddFileToHistory(filename)

    def FileHistory(self, filenum):
        path = self.filehistory.GetHistoryFile(filenum)
        self._ExecuteScript(path)

    def CheckForExit(self, command, more):
        if command in ["exit", "quit", "close"]:
            wx.CallAfter(self.frame.Close)

    def OpenShell(self):
        shellframe = wx.py.shell.ShellFrame(parent=self.frame, title="VISAN shell", locals=self.shell.interp.locals)
        shellframe.SetSize((700, 500))
        shellframe.Show()
        return True

    def OpenCrust(self):
        shellframe = wx.py.crust.CrustFrame(parent=self.frame, title="VISAN Crust", locals=self.shell.interp.locals)
        shellframe.SetSize((700, 500))
        shellframe.Show()
        return True

    def _CreateSplashScreen(self, timeout=True):
        self.aboutIsShown = True
        mode = wx.adv.SPLASH_CENTRE_ON_PARENT
        if timeout:
            mode |= wx.adv.SPLASH_TIMEOUT
        else:
            mode |= wx.adv.SPLASH_NO_TIMEOUT
        splashscreen = wx.adv.SplashScreen(wx.Bitmap(os.path.join(self.datadir, 'visan-logo.png'), wx.BITMAP_TYPE_PNG),
                                           mode, 750, self.frame, -1)
        splashscreen.Bind(wx.EVT_CLOSE, self.CloseAbout)
        wx.Yield()

    def ShowAbout(self):
        if self.aboutIsShown:
            return
        self._CreateSplashScreen(False)

    def CloseAbout(self, event):
        self.aboutIsShown = False
        event.Skip()

    def Exit(self):
        self._FileHistoryToPrefs()
        config = wx.Config.Get()
        config.WriteBool('CODA/PerformConversions', coda.get_option_perform_conversions())
        config.WriteBool('CODA/FilterRecordFields', coda.get_option_filter_record_fields())
        config.DeleteEntry('DirectoryLocation/ApplicationData')
        config.DeleteEntry('IconFile')
        config.Flush()
        sys.exit(0)

    def ShowIntro(self):
        try:
            if self.introframe.IsIconized():
                self.introframe.Iconize(False)
            if not self.introframe.IsShown():
                self.introframe.Show(True)
            self.introframe.Raise()
        except (AttributeError, RuntimeError):
            self.introframe = IntroFrame(self.frame)
            self.introframe.Show(True)

    def ShowHelp(self, mode):
        if mode == "coda":
            helpurl = "http://stcorp.github.io/coda/doc/html/index.html"
        elif mode == "harp":
            helpurl = "http://stcorp.github.io/harp/doc/html/index.html"
        elif mode == "codadef":
            helpurl = "https://stcorp.github.io/codadef-documentation/"
        elif mode == "python":
            helpurl = "http://docs.python.org/%d.%d/" % (sys.version_info.major, sys.version_info.minor)
        elif mode == "numpy":
            helpurl = "http://docs.scipy.org/doc/numpy/reference/"
        else:
            helpurl = "http://stcorp.github.io/visan/"

        try:
            webbrowser.open_new(helpurl)
        except Error as ex:
            wx.MessageBox("Error launching external browser:\n%s" % str(ex),
                          "VISAN Error", wx.OK | wx.ICON_INFORMATION)


class LogWindow(wx.Frame):

    def __init__(self, parent, id, title, pos=wx.DefaultPosition, size=(650, 400)):
        wx.Frame.__init__(self, parent, id, title, pos, size)
        if wx.Config.Get().Read("IconFile"):
            self.SetIcon(wx.Icon(wx.Config.Get().Read("IconFile")))
        self.Bind(wx.EVT_CLOSE, self.OnClose)

    def OnClose(self, event):
        if event.CanVeto():
            self.Show(False)
            event.Veto()
        else:
            self.Destroy()


class SessionListing(wx.TextCtrl):
    """Text control containing all commands for session."""

    def __init__(self, parent=None, id=-1):
        style = wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_RICH | wx.TE_DONTWRAP
        wx.TextCtrl.__init__(self, parent=parent, id=id, style=style)

    def Start(self):
        wx.py.dispatcher.connect(receiver=self.Push, signal='Interpreter.push')

    def Stop(self):
        wx.py.dispatcher.disconnect(receiver=self.Push, signal='Interpreter.push')

    def Push(self, command, more):
        """Receiver for Interpreter.push signal."""
        if command and not more:
            self.SetInsertionPointEnd()
            (start, end) = self.GetSelection()
            if start != end:
                self.SetSelection(0, 0)
            self.AppendText(command + '\n')



# This is an example VISAN script for the gammap() function.

# This example shows the gammap function (= gamma(a,x) / Gamma(a)) for
# several values of 'a'.

from visan.math import gammap


def run():

    x = numpy.arange(1500.) / 100
    x = x[1:]
    a = numpy.ones(numpy.shape(x), dtype=float)
    y = gammap(0.5 * a, x)
    w = plot(x, y, name='a=0.5')
    y = gammap(1.0 * a, x)
    plot(x, y, name='a=1.0', window=w)
    y = gammap(3.0 * a, x)
    plot(x, y, name='a=3.0', window=w)
    y = gammap(10.0 * a, x)
    plot(x, y, xrange=(0, 15), yrange=(0, 1), name='a=10.0', window=w, title='gammap() example')


run()
